@@ -18,9 +18,14 @@ from pcs.lib.commands import (
     acl,
     alert,
     booth,
+    cluster,
+    fencing_topology,
+    node,
     qdevice,
     quorum,
     resource_agent,
+    resource,
+    stonith,
     sbd,
     stonith_agent,
 )
@@ -49,6 +54,7 @@ def cli_env_to_lib_env(cli_env):
         booth=cli_env.booth,
         auth_tokens_getter=cli_env.auth_tokens_getter,
         cluster_conf_data=cli_env.cluster_conf_data,
+        request_timeout=cli_env.request_timeout,
     )
 
 def lib_env_to_cli_env(lib_env, cli_env):
@@ -176,6 +182,22 @@ def load_module(env, middleware_factory, name):
             }
         )
 
+    if name == "cluster":
+        return bind_all(
+            env,
+            middleware.build(
+                middleware_factory.cib,
+                middleware_factory.corosync_conf_existing,
+            ),
+            {
+                "node_add_remote": cluster.node_add_remote,
+                "node_add_guest": cluster.node_add_guest,
+                "node_remove_remote": cluster.node_remove_remote,
+                "node_remove_guest": cluster.node_remove_guest,
+                "node_clear": cluster.node_clear,
+            }
+        )
+
     if name == 'constraint_colocation':
         return bind_all(
             env,
@@ -208,6 +230,37 @@ def load_module(env, middleware_factory, name):
             }
         )
 
+    if name == "fencing_topology":
+        return bind_all(
+            env,
+            middleware.build(middleware_factory.cib),
+            {
+                "add_level": fencing_topology.add_level,
+                "get_config": fencing_topology.get_config,
+                "remove_all_levels": fencing_topology.remove_all_levels,
+                "remove_levels_by_params":
+                    fencing_topology.remove_levels_by_params,
+                "verify": fencing_topology.verify,
+            }
+        )
+
+    if name == "node":
+        return bind_all(
+            env,
+            middleware.build(middleware_factory.cib),
+            {
+                "maintenance_unmaintenance_all":
+                    node.maintenance_unmaintenance_all,
+                "maintenance_unmaintenance_list":
+                    node.maintenance_unmaintenance_list,
+                "maintenance_unmaintenance_local":
+                    node.maintenance_unmaintenance_local,
+                "standby_unstandby_all": node.standby_unstandby_all,
+                "standby_unstandby_list": node.standby_unstandby_list,
+                "standby_unstandby_local": node.standby_unstandby_local,
+            }
+        )
+
     if name == "qdevice":
         return bind_all(
             env,
@@ -261,6 +314,41 @@ def load_module(env, middleware_factory, name):
             }
         )
 
+    if name == "resource":
+        return bind_all(
+            env,
+            middleware.build(
+                middleware_factory.cib,
+                middleware_factory.corosync_conf_existing,
+            ),
+            {
+                "bundle_create": resource.bundle_create,
+                "bundle_update": resource.bundle_update,
+                "create": resource.create,
+                "create_as_master": resource.create_as_master,
+                "create_as_clone": resource.create_as_clone,
+                "create_in_group": resource.create_in_group,
+                "create_into_bundle": resource.create_into_bundle,
+                "disable": resource.disable,
+                "enable": resource.enable,
+                "manage": resource.manage,
+                "unmanage": resource.unmanage,
+            }
+        )
+    if name == "stonith":
+        return bind_all(
+            env,
+            middleware.build(
+                middleware_factory.cib,
+                middleware_factory.corosync_conf_existing,
+            ),
+            {
+                "create": stonith.create,
+                "create_in_group": stonith.create_in_group,
+            }
+        )
+
+
     if name == "sbd":
         return bind_all(
             env,
@@ -271,6 +359,9 @@ def load_module(env, middleware_factory, name):
                 "get_cluster_sbd_status": sbd.get_cluster_sbd_status,
                 "get_cluster_sbd_config": sbd.get_cluster_sbd_config,
                 "get_local_sbd_config": sbd.get_local_sbd_config,
+                "initialize_block_devices": sbd.initialize_block_devices,
+                "get_local_devices_info": sbd.get_local_devices_info,
+                "set_message": sbd.set_message,
             }
         )
 
