@@ -5,7 +5,7 @@ from __future__ import (
     unicode_literals,
 )
 
-from unittest import TestCase
+from pcs.test.tools.pcs_unittest import TestCase
 from pcs.cli.constraint_ticket import console_report
 
 class ConstraintPlainTest(TestCase):
