@@ -10,15 +10,22 @@ FORCE_ACTIVE_RRP = "ACTIVE_RRP"
 FORCE_ALERT_RECIPIENT_VALUE_NOT_UNIQUE = "FORCE_ALERT_RECIPIENT_VALUE_NOT_UNIQUE"
 FORCE_BOOTH_DESTROY = "FORCE_BOOTH_DESTROY"
 FORCE_BOOTH_REMOVE_FROM_CIB = "FORCE_BOOTH_REMOVE_FROM_CIB"
+FORCE_REMOVE_MULTIPLE_NODES = "FORCE_REMOVE_MULTIPLE_NODES"
 FORCE_CONSTRAINT_DUPLICATE = "CONSTRAINT_DUPLICATE"
 FORCE_CONSTRAINT_MULTIINSTANCE_RESOURCE = "CONSTRAINT_MULTIINSTANCE_RESOURCE"
 FORCE_FILE_OVERWRITE = "FORCE_FILE_OVERWRITE"
 FORCE_LOAD_THRESHOLD = "LOAD_THRESHOLD"
 FORCE_METADATA_ISSUE = "METADATA_ISSUE"
+FORCE_NODE_DOES_NOT_EXIST = "FORCE_NODE_DOES_NOT_EXIST"
 FORCE_OPTIONS = "OPTIONS"
 FORCE_QDEVICE_MODEL = "QDEVICE_MODEL"
 FORCE_QDEVICE_USED = "QDEVICE_USED"
+FORCE_STONITH_RESOURCE_DOES_NOT_EXIST = "FORCE_STONITH_RESOURCE_DOES_NOT_EXIST"
+FORCE_NOT_SUITABLE_COMMAND = "FORCE_NOT_SUITABLE_COMMAND"
+FORCE_CLEAR_CLUSTER_NODE = "FORCE_CLEAR_CLUSTER_NODE"
 SKIP_OFFLINE_NODES = "SKIP_OFFLINE_NODES"
+SKIP_FILE_DISTRIBUTION_ERRORS = "SKIP_FILE_DISTRIBUTION_ERRORS"
+SKIP_ACTION_ON_NODES_ERRORS = "SKIP_ACTION_ON_NODES_ERRORS"
 SKIP_UNREADABLE_CONFIG = "SKIP_UNREADABLE_CONFIG"
 
 AGENT_NAME_GUESS_FOUND_MORE_THAN_ONE = "AGENT_NAME_GUESS_FOUND_MORE_THAN_ONE"
@@ -56,14 +63,16 @@ BOOTH_UNSUPORTED_FILE_LOCATION = "BOOTH_UNSUPORTED_FILE_LOCATION"
 CIB_ACL_ROLE_IS_ALREADY_ASSIGNED_TO_TARGET = "CIB_ACL_ROLE_IS_ALREADY_ASSIGNED_TO_TARGET"
 CIB_ACL_ROLE_IS_NOT_ASSIGNED_TO_TARGET = "CIB_ACL_ROLE_IS_NOT_ASSIGNED_TO_TARGET"
 CIB_ACL_TARGET_ALREADY_EXISTS = "CIB_ACL_TARGET_ALREADY_EXISTS"
-CIB_ALERT_NOT_FOUND = "CIB_ALERT_NOT_FOUND"
 CIB_ALERT_RECIPIENT_ALREADY_EXISTS = "CIB_ALERT_RECIPIENT_ALREADY_EXISTS"
 CIB_ALERT_RECIPIENT_VALUE_INVALID = "CIB_ALERT_RECIPIENT_VALUE_INVALID"
 CIB_CANNOT_FIND_MANDATORY_SECTION = "CIB_CANNOT_FIND_MANDATORY_SECTION"
+CIB_FENCING_LEVEL_ALREADY_EXISTS = "CIB_FENCING_LEVEL_ALREADY_EXISTS"
+CIB_FENCING_LEVEL_DOES_NOT_EXIST = "CIB_FENCING_LEVEL_DOES_NOT_EXIST"
 CIB_LOAD_ERROR_BAD_FORMAT = "CIB_LOAD_ERROR_BAD_FORMAT"
 CIB_LOAD_ERROR = "CIB_LOAD_ERROR"
 CIB_LOAD_ERROR_SCOPE_MISSING = "CIB_LOAD_ERROR_SCOPE_MISSING"
 CIB_PUSH_ERROR = "CIB_PUSH_ERROR"
+CIB_SAVE_TMP_ERROR = "CIB_SAVE_TMP_ERROR"
 CIB_UPGRADE_FAILED = "CIB_UPGRADE_FAILED"
 CIB_UPGRADE_FAILED_TO_MINIMAL_REQUIRED_VERSION = "CIB_UPGRADE_FAILED_TO_MINIMAL_REQUIRED_VERSION"
 CIB_UPGRADE_SUCCESSFUL = "CIB_UPGRADE_SUCCESSFUL"
@@ -76,6 +85,7 @@ CMAN_UNSUPPORTED_COMMAND = "CMAN_UNSUPPORTED_COMMAND"
 COMMON_ERROR = 'COMMON_ERROR'
 COMMON_INFO = 'COMMON_INFO'
 LIVE_ENVIRONMENT_REQUIRED = "LIVE_ENVIRONMENT_REQUIRED"
+LIVE_ENVIRONMENT_REQUIRED_FOR_LOCAL_NODE = "LIVE_ENVIRONMENT_REQUIRED_FOR_LOCAL_NODE"
 COROSYNC_CONFIG_ACCEPTED_BY_NODE = "COROSYNC_CONFIG_ACCEPTED_BY_NODE"
 COROSYNC_CONFIG_DISTRIBUTION_STARTED = "COROSYNC_CONFIG_DISTRIBUTION_STARTED"
 COROSYNC_CONFIG_DISTRIBUTION_NODE_ERROR = "COROSYNC_CONFIG_DISTRIBUTION_NODE_ERROR"
@@ -90,40 +100,64 @@ COROSYNC_QUORUM_GET_STATUS_ERROR = "COROSYNC_QUORUM_GET_STATUS_ERROR"
 COROSYNC_QUORUM_SET_EXPECTED_VOTES_ERROR = "COROSYNC_QUORUM_SET_EXPECTED_VOTES_ERROR"
 COROSYNC_RUNNING_ON_NODE = "COROSYNC_RUNNING_ON_NODE"
 CRM_MON_ERROR = "CRM_MON_ERROR"
+DEPRECATED_OPTION = "DEPRECATED_OPTION"
 DUPLICATE_CONSTRAINTS_EXIST = "DUPLICATE_CONSTRAINTS_EXIST"
 EMPTY_RESOURCE_SET_LIST = "EMPTY_RESOURCE_SET_LIST"
 EMPTY_ID = "EMPTY_ID"
 FILE_ALREADY_EXISTS = "FILE_ALREADY_EXISTS"
 FILE_DOES_NOT_EXIST = "FILE_DOES_NOT_EXIST"
 FILE_IO_ERROR = "FILE_IO_ERROR"
+FILES_DISTRIBUTION_STARTED = "FILES_DISTRIBUTION_STARTED"
+FILE_DISTRIBUTION_ERROR = "FILE_DISTRIBUTION_ERROR"
+FILE_DISTRIBUTION_SUCCESS = "FILE_DISTRIBUTION_SUCCESS"
+FILES_REMOVE_FROM_NODE_STARTED = "FILES_REMOVE_FROM_NODE_STARTED"
+FILE_REMOVE_FROM_NODE_ERROR = "FILE_REMOVE_FROM_NODE_ERROR"
+FILE_REMOVE_FROM_NODE_SUCCESS = "FILE_REMOVE_FROM_NODE_SUCCESS"
 ID_ALREADY_EXISTS = 'ID_ALREADY_EXISTS'
+ID_BELONGS_TO_UNEXPECTED_TYPE = "ID_BELONGS_TO_UNEXPECTED_TYPE"
 ID_NOT_FOUND = 'ID_NOT_FOUND'
 IGNORED_CMAN_UNSUPPORTED_OPTION = 'IGNORED_CMAN_UNSUPPORTED_OPTION'
 INVALID_ID = "INVALID_ID"
 INVALID_OPTION = "INVALID_OPTION"
+INVALID_OPTION_TYPE = "INVALID_OPTION_TYPE"
 INVALID_OPTION_VALUE = "INVALID_OPTION_VALUE"
 INVALID_RESOURCE_NAME = 'INVALID_RESOURCE_NAME'
 INVALID_RESOURCE_AGENT_NAME = 'INVALID_RESOURCE_AGENT_NAME'
 INVALID_RESPONSE_FORMAT = "INVALID_RESPONSE_FORMAT"
 INVALID_SCORE = "INVALID_SCORE"
+INVALID_STONITH_AGENT_NAME = "INVALID_STONITH_AGENT_NAME"
 INVALID_TIMEOUT_VALUE = "INVALID_TIMEOUT_VALUE"
 MULTIPLE_SCORE_OPTIONS = "MULTIPLE_SCORE_OPTIONS"
+MULTIPLE_RESULTS_FOUND = "MULTIPLE_RESULTS_FOUND"
+MUTUALLY_EXCLUSIVE_OPTIONS = "MUTUALLY_EXCLUSIVE_OPTIONS"
+CANNOT_ADD_NODE_IS_IN_CLUSTER = "CANNOT_ADD_NODE_IS_IN_CLUSTER"
+CANNOT_ADD_NODE_IS_RUNNING_SERVICE = "CANNOT_ADD_NODE_IS_RUNNING_SERVICE"
 NODE_COMMUNICATION_COMMAND_UNSUCCESSFUL = "NODE_COMMUNICATION_COMMAND_UNSUCCESSFUL"
+NODE_COMMUNICATION_DEBUG_INFO = "NODE_COMMUNICATION_DEBUG_INFO"
 NODE_COMMUNICATION_ERROR = "NODE_COMMUNICATION_ERROR"
 NODE_COMMUNICATION_ERROR_NOT_AUTHORIZED = "NODE_COMMUNICATION_ERROR_NOT_AUTHORIZED"
 NODE_COMMUNICATION_ERROR_PERMISSION_DENIED = "NODE_COMMUNICATION_ERROR_PERMISSION_DENIED"
 NODE_COMMUNICATION_ERROR_UNABLE_TO_CONNECT = "NODE_COMMUNICATION_ERROR_UNABLE_TO_CONNECT"
 NODE_COMMUNICATION_ERROR_UNSUPPORTED_COMMAND = "NODE_COMMUNICATION_ERROR_UNSUPPORTED_COMMAND"
+NODE_COMMUNICATION_ERROR_TIMED_OUT = "NODE_COMMUNICATION_ERROR_TIMED_OUT"
 NODE_COMMUNICATION_FINISHED = "NODE_COMMUNICATION_FINISHED"
 NODE_COMMUNICATION_NOT_CONNECTED = "NODE_COMMUNICATION_NOT_CONNECTED"
+NODE_COMMUNICATION_PROXY_IS_SET = "NODE_COMMUNICATION_PROXY_IS_SET"
 NODE_COMMUNICATION_STARTED = "NODE_COMMUNICATION_STARTED"
 NODE_NOT_FOUND = "NODE_NOT_FOUND"
+NODE_REMOVE_IN_PACEMAKER_FAILED = "NODE_REMOVE_IN_PACEMAKER_FAILED"
 NON_UDP_TRANSPORT_ADDR_MISMATCH = 'NON_UDP_TRANSPORT_ADDR_MISMATCH'
+NOLIVE_SKIP_FILES_DISTRIBUTION="NOLIVE_SKIP_FILES_DISTRIBUTION"
+NOLIVE_SKIP_FILES_REMOVE="NOLIVE_SKIP_FILES_REMOVE"
+NOLIVE_SKIP_SERVICE_COMMAND_ON_NODES="NOLIVE_SKIP_SERVICE_COMMAND_ON_NODES"
+NODE_TO_CLEAR_IS_STILL_IN_CLUSTER = "NODE_TO_CLEAR_IS_STILL_IN_CLUSTER"
 OMITTING_NODE = "OMITTING_NODE"
+OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT = "OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT"
 PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND = "PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND"
 PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE"
 PARSE_ERROR_COROSYNC_CONF = "PARSE_ERROR_COROSYNC_CONF"
 PARSE_ERROR_COROSYNC_CONF_UNEXPECTED_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_UNEXPECTED_CLOSING_BRACE"
+PREREQUISITE_OPTION_IS_MISSING = "PREREQUISITE_OPTION_IS_MISSING"
 QDEVICE_ALREADY_DEFINED = "QDEVICE_ALREADY_DEFINED"
 QDEVICE_ALREADY_INITIALIZED = "QDEVICE_ALREADY_INITIALIZED"
 QDEVICE_CERTIFICATE_ACCEPTED_BY_NODE = "QDEVICE_CERTIFICATE_ACCEPTED_BY_NODE"
@@ -144,26 +178,43 @@ QDEVICE_CLIENT_RELOAD_STARTED = "QDEVICE_CLIENT_RELOAD_STARTED"
 QDEVICE_REMOVE_OR_CLUSTER_STOP_NEEDED = "QDEVICE_REMOVE_OR_CLUSTER_STOP_NEEDED"
 QDEVICE_USED_BY_CLUSTERS = "QDEVICE_USED_BY_CLUSTERS"
 REQUIRED_OPTION_IS_MISSING = "REQUIRED_OPTION_IS_MISSING"
+REQUIRED_OPTION_OF_ALTERNATIVES_IS_MISSING = "REQUIRED_OPTION_OF_ALTERNATIVES_IS_MISSING"
+RESOURCE_BUNDLE_ALREADY_CONTAINS_A_RESOURCE = "RESOURCE_BUNDLE_ALREADY_CONTAINS_A_RESOURCE"
+RESOURCE_CANNOT_BE_NEXT_TO_ITSELF_IN_GROUP = "RESOURCE_CANNOT_BE_NEXT_TO_ITSELF_IN_GROUP"
 RESOURCE_CLEANUP_ERROR = "RESOURCE_CLEANUP_ERROR"
 RESOURCE_CLEANUP_TOO_TIME_CONSUMING = 'RESOURCE_CLEANUP_TOO_TIME_CONSUMING'
-RESOURCE_DOES_NOT_EXIST = 'RESOURCE_DOES_NOT_EXIST'
+RESOURCE_DOES_NOT_RUN = "RESOURCE_DOES_NOT_RUN"
 RESOURCE_FOR_CONSTRAINT_IS_MULTIINSTANCE = 'RESOURCE_FOR_CONSTRAINT_IS_MULTIINSTANCE'
-RESOURCE_WAIT_ERROR = "RESOURCE_WAIT_ERROR"
-RESOURCE_WAIT_NOT_SUPPORTED = "RESOURCE_WAIT_NOT_SUPPORTED"
-RESOURCE_WAIT_TIMED_OUT = "RESOURCE_WAIT_TIMED_OUT"
+RESOURCE_IS_GUEST_NODE_ALREADY = "RESOURCE_IS_GUEST_NODE_ALREADY"
+RESOURCE_IS_UNMANAGED = "RESOURCE_IS_UNMANAGED"
+RESOURCE_MANAGED_NO_MONITOR_ENABLED = "RESOURCE_MANAGED_NO_MONITOR_ENABLED"
+RESOURCE_OPERATION_INTERVAL_DUPLICATION = "RESOURCE_OPERATION_INTERVAL_DUPLICATION"
+RESOURCE_OPERATION_INTERVAL_ADAPTED = "RESOURCE_OPERATION_INTERVAL_ADAPTED"
+RESOURCE_RUNNING_ON_NODES = "RESOURCE_RUNNING_ON_NODES"
 RRP_ACTIVE_NOT_SUPPORTED = 'RRP_ACTIVE_NOT_SUPPORTED'
 RUN_EXTERNAL_PROCESS_ERROR = "RUN_EXTERNAL_PROCESS_ERROR"
 RUN_EXTERNAL_PROCESS_FINISHED = "RUN_EXTERNAL_PROCESS_FINISHED"
 RUN_EXTERNAL_PROCESS_STARTED = "RUN_EXTERNAL_PROCESS_STARTED"
 SBD_CHECK_STARTED = "SBD_CHECK_STARTED"
 SBD_CHECK_SUCCESS = "SBD_CHECK_SUCCESS"
-SBD_CONFIG_DISTRIBUTION_STARTED = "SBD_CONFIG_DISTRIBUTION_STARTED"
 SBD_CONFIG_ACCEPTED_BY_NODE = "SBD_CONFIG_ACCEPTED_BY_NODE"
+SBD_CONFIG_DISTRIBUTION_STARTED = "SBD_CONFIG_DISTRIBUTION_STARTED"
+SBD_DEVICE_DOES_NOT_EXIST = "SBD_DEVICE_DOES_NOT_EXIST"
+SBD_DEVICE_DUMP_ERROR = "SBD_DEVICE_DUMP_ERROR"
+SBD_DEVICE_INITIALIZATION_ERROR = "SBD_DEVICE_INITIALIZATION_ERROR"
+SBD_DEVICE_INITIALIZATION_STARTED = "SBD_DEVICE_INITIALIZATION_STARTED"
+SBD_DEVICE_INITIALIZATION_SUCCESS = "SBD_DEVICE_INITIALIZATION_SUCCESS"
+SBD_DEVICE_IS_NOT_BLOCK_DEVICE = "SBD_DEVICE_IS_NOT_BLOCK_DEVICE"
+SBD_DEVICE_LIST_ERROR = "SBD_DEVICE_LIST_ERROR"
+SBD_DEVICE_MESSAGE_ERROR = "SBD_DEVICE_MESSAGE_ERROR"
+SBD_DEVICE_PATH_NOT_ABSOLUTE = "SBD_DEVICE_PATH_NOT_ABSOLUTE"
 SBD_DISABLING_STARTED = "SBD_DISABLING_STARTED"
 SBD_ENABLING_STARTED = "SBD_ENABLING_STARTED"
-SBD_NOT_INSTALLED = "SBD_NOT_INSTALLED"
+SBD_NO_DEVICE_FOR_NODE = "SBD_NO_DEVICE_FOR_NODE"
 SBD_NOT_ENABLED = "SBD_NOT_ENABLED"
+SBD_NOT_INSTALLED = "SBD_NOT_INSTALLED"
 SBD_REQUIRES_ATB = "SBD_REQUIRES_ATB"
+SBD_TOO_MANY_DEVICES_FOR_NODE = "SBD_TOO_MANY_DEVICES_FOR_NODE"
 SERVICE_DISABLE_ERROR = "SERVICE_DISABLE_ERROR"
 SERVICE_DISABLE_STARTED = "SERVICE_DISABLE_STARTED"
 SERVICE_DISABLE_SUCCESS = "SERVICE_DISABLE_SUCCESS"
@@ -180,6 +231,10 @@ SERVICE_START_SUCCESS = "SERVICE_START_SUCCESS"
 SERVICE_STOP_ERROR = "SERVICE_STOP_ERROR"
 SERVICE_STOP_STARTED = "SERVICE_STOP_STARTED"
 SERVICE_STOP_SUCCESS = "SERVICE_STOP_SUCCESS"
+STONITH_RESOURCES_DO_NOT_EXIST = "STONITH_RESOURCES_DO_NOT_EXIST"
+SERVICE_COMMANDS_ON_NODES_STARTED = "SERVICE_COMMANDS_ON_NODES_STARTED"
+SERVICE_COMMAND_ON_NODE_ERROR = "SERVICE_COMMAND_ON_NODE_ERROR"
+SERVICE_COMMAND_ON_NODE_SUCCESS = "SERVICE_COMMAND_ON_NODE_SUCCESS"
 UNABLE_TO_DETERMINE_USER_UID = "UNABLE_TO_DETERMINE_USER_UID"
 UNABLE_TO_DETERMINE_GROUP_GID = "UNABLE_TO_DETERMINE_GROUP_GID"
 UNABLE_TO_GET_AGENT_METADATA = 'UNABLE_TO_GET_AGENT_METADATA'
@@ -189,4 +244,11 @@ UNABLE_TO_GET_SBD_STATUS = "UNABLE_TO_GET_SBD_STATUS"
 UNKNOWN_COMMAND = 'UNKNOWN_COMMAND'
 WATCHDOG_INVALID = "WATCHDOG_INVALID"
 UNSUPPORTED_OPERATION_ON_NON_SYSTEMD_SYSTEMS = "UNSUPPORTED_OPERATION_ON_NON_SYSTEMD_SYSTEMS"
+USE_COMMAND_NODE_ADD_REMOTE = "USE_COMMAND_NODE_ADD_REMOTE"
+USE_COMMAND_NODE_ADD_GUEST = "USE_COMMAND_NODE_ADD_GUEST"
+USE_COMMAND_NODE_REMOVE_GUEST = "USE_COMMAND_NODE_REMOVE_GUEST"
+WAIT_FOR_IDLE_ERROR = "WAIT_FOR_IDLE_ERROR"
+WAIT_FOR_IDLE_NOT_LIVE_CLUSTER = "WAIT_FOR_IDLE_NOT_LIVE_CLUSTER"
+WAIT_FOR_IDLE_NOT_SUPPORTED = "WAIT_FOR_IDLE_NOT_SUPPORTED"
+WAIT_FOR_IDLE_TIMED_OUT = "WAIT_FOR_IDLE_TIMED_OUT"
 WATCHDOG_NOT_FOUND = "WATCHDOG_NOT_FOUND"
