@@ -17,8 +17,10 @@ from xml.dom.minidom import parse
 import logging
 import pwd
 import grp
+import tempfile
 import time
 import platform
+import shutil
 
 try:
     import clufter.facts
@@ -122,20 +124,26 @@ def config_show(argv):
             utils.process_library_reports(e.args)
 
 def config_show_cib():
+    lib = utils.get_library_wrapper()
+    modificators = utils.get_modificators()
+
     print("Resources:")
     utils.pcs_options["--all"] = 1
     utils.pcs_options["--full"] = 1
     resource.resource_show([])
+
     print()
     print("Stonith Devices:")
     resource.resource_show([], True)
     print("Fencing Levels:")
-    stonith.stonith_level_show()
-    print()
+    levels = stonith.stonith_level_config_to_str(
+        lib.fencing_topology.get_config()
+    )
+    if levels:
+        print("\n".join(indent(levels, 2)))
 
-    lib = utils.get_library_wrapper()
+    print()
     constraint.location_show([])
-    modificators = utils.get_modificators()
     order_command.show(lib, [], modificators)
     colocation_command.show(lib, [], modificators)
     ticket_command.show(lib, [], modificators)
@@ -341,6 +349,7 @@ def config_restore_local(infile_name, infile_obj):
     file_list = config_backup_path_list(with_uid_gid=True)
     tarball_file_list = []
     version = None
+    tmp_dir = None
     try:
         tarball = tarfile.open(infile_name, "r|*", infile_obj)
         while True:
@@ -387,15 +396,30 @@ def config_restore_local(infile_name, infile_obj):
                 path = os.path.dirname(path)
             if not extract_info:
                 continue
-            path_extract = os.path.dirname(extract_info["path"])
-            tarball.extractall(path_extract, [tar_member_info])
-            path_full = os.path.join(path_extract, tar_member_info.name)
+            path_full = None
+            if hasattr(extract_info.get("pre_store_call"), '__call__'):
+                extract_info["pre_store_call"]()
+            if "rename" in extract_info and extract_info["rename"]:
+                if tmp_dir is None:
+                    tmp_dir = tempfile.mkdtemp()
+                tarball.extractall(tmp_dir, [tar_member_info])
+                path_full = extract_info["path"]
+                os.rename(
+                    os.path.join(tmp_dir, tar_member_info.name), path_full
+                )
+            else:
+                dir_path = os.path.dirname(extract_info["path"])
+                tarball.extractall(dir_path, [tar_member_info])
+                path_full = os.path.join(dir_path, tar_member_info.name)
             file_attrs = extract_info["attrs"]
             os.chmod(path_full, file_attrs["mode"])
             os.chown(path_full, file_attrs["uid"], file_attrs["gid"])
         tarball.close()
-    except (tarfile.TarError, EnvironmentError) as e:
+    except (tarfile.TarError, EnvironmentError, OSError) as e:
         utils.err("unable to restore the cluster: %s" % e)
+    finally:
+        if tmp_dir:
+            shutil.rmtree(tmp_dir, ignore_errors=True)
 
     try:
         sig_path = os.path.join(settings.cib_dir, "cib.xml.sig")
@@ -414,6 +438,8 @@ def config_backup_path_list(with_uid_gid=False, force_rhel6=None):
         "uid": 0,
         "gid": 0,
     }
+    corosync_authkey_attrs = dict(corosync_attrs)
+    corosync_authkey_attrs["mode"] = 0o400
     cib_attrs = {
         "mtime": int(time.time()),
         "mode": 0o600,
@@ -421,25 +447,32 @@ def config_backup_path_list(with_uid_gid=False, force_rhel6=None):
         "gname": settings.pacemaker_gname,
     }
     if with_uid_gid:
-        try:
-            cib_attrs["uid"] = pwd.getpwnam(cib_attrs["uname"]).pw_uid
-        except KeyError:
-            utils.err(
-                "Unable to determine uid of user '%s'" % cib_attrs["uname"]
-            )
-        try:
-            cib_attrs["gid"] = grp.getgrnam(cib_attrs["gname"]).gr_gid
-        except KeyError:
-            utils.err(
-                "Unable to determine gid of group '%s'" % cib_attrs["gname"]
-            )
+        cib_attrs["uid"] = _get_uid(cib_attrs["uname"])
+        cib_attrs["gid"] = _get_gid(cib_attrs["gname"])
 
+    pcmk_authkey_attrs = dict(cib_attrs)
+    pcmk_authkey_attrs["mode"] = 0o440
     file_list = {
         "cib.xml": {
             "path": os.path.join(settings.cib_dir, "cib.xml"),
             "required": True,
             "attrs": dict(cib_attrs),
         },
+        "corosync_authkey": {
+            "path": settings.corosync_authkey_file,
+            "required": False,
+            "attrs": corosync_authkey_attrs,
+            "restore_procedure": None,
+            "rename": True,
+        },
+        "pacemaker_authkey": {
+            "path": settings.pacemaker_authkey_file,
+            "required": False,
+            "attrs": pcmk_authkey_attrs,
+            "restore_procedure": None,
+            "rename": True,
+            "pre_store_call": _ensure_etc_pacemaker_exists,
+        },
     }
     if rhel6:
         file_list["cluster.conf"] = {
@@ -472,6 +505,35 @@ def config_backup_path_list(with_uid_gid=False, force_rhel6=None):
         }
     return file_list
 
+
+def _get_uid(user_name):
+    try:
+        return pwd.getpwnam(user_name).pw_uid
+    except KeyError:
+        utils.err("Unable to determine uid of user '{0}'".format(user_name))
+
+
+def _get_gid(group_name):
+    try:
+        return grp.getgrnam(group_name).gr_gid
+    except KeyError:
+        utils.err(
+            "Unable to determine gid of group '{0}'".format(group_name)
+        )
+
+
+def _ensure_etc_pacemaker_exists():
+    dir_name = os.path.dirname(settings.pacemaker_authkey_file)
+    if not os.path.exists(dir_name):
+        os.mkdir(dir_name)
+        os.chmod(dir_name, 0o750)
+        os.chown(
+            dir_name,
+            _get_uid(settings.pacemaker_uname),
+            _get_gid(settings.pacemaker_gname)
+        )
+
+
 def config_backup_check_version(version):
     try:
         version_number = int(version)
@@ -621,8 +683,6 @@ def config_import_cman(argv):
         "batch": True,
         "sys": "linux",
         "dist": dist,
-        # Make it work on RHEL6 as well for sure
-        "color": "always" if sys.stdout.isatty() else "never"
     }
     if interactive:
         if "EDITOR" not in os.environ:
@@ -670,7 +730,7 @@ def config_import_cman(argv):
     if output_format in ("pcs-commands", "pcs-commands-verbose"):
         ok, message = utils.write_file(
             dry_run_output,
-            clufter_args_obj.output["passout"]
+            clufter_args_obj.output["passout"].decode()
         )
         if not ok:
             utils.err(message)
@@ -692,14 +752,14 @@ def config_import_cman(argv):
         config_backup_add_version_to_tarball(tarball)
         utils.tar_add_file_data(
             tarball,
-            clufter_args_obj.cib["passout"].encode("utf-8"),
+            clufter_args_obj.cib["passout"],
             "cib.xml",
             **file_list["cib.xml"]["attrs"]
         )
         if output_format == "cluster.conf":
             utils.tar_add_file_data(
                 tarball,
-                clufter_args_obj.ccs_pcmk["passout"].encode("utf-8"),
+                clufter_args_obj.ccs_pcmk["passout"],
                 "cluster.conf",
                 **file_list["cluster.conf"]["attrs"]
             )
@@ -720,7 +780,7 @@ def config_import_cman(argv):
             )("bytestring")
             utils.tar_add_file_data(
                 tarball,
-                corosync_conf_data.encode("utf-8"),
+                corosync_conf_data,
                 "corosync.conf",
                 **file_list["corosync.conf"]["attrs"]
             )
@@ -738,7 +798,7 @@ def config_import_cman(argv):
                 )("bytestring")
                 utils.tar_add_file_data(
                     tarball,
-                    uidgid_data.encode("utf-8"),
+                    uidgid_data,
                     "uidgid.d/" + filename,
                     **file_list["uidgid.d"]["attrs"]
                 )
@@ -796,8 +856,6 @@ def config_export_pcs_commands(argv, verbose=False):
         "batch": True,
         "sys": "linux",
         "dist": dist,
-        # Make it work on RHEL6 as well for sure
-        "color": "always" if sys.stdout.isatty() else "never",
         "coro": settings.corosync_conf_file,
         "ccs": settings.cluster_conf_file,
         "start_wait": "60",
@@ -839,7 +897,7 @@ def config_export_pcs_commands(argv, verbose=False):
     if output_file:
         ok, message = utils.write_file(
             output_file,
-            clufter_args_obj.output["passout"]
+            clufter_args_obj.output["passout"].decode()
         )
         if not ok:
             utils.err(message)
