@@ -8,11 +8,9 @@ from __future__ import (
 from lxml import etree
 
 from pcs.lib import reports
-from pcs.lib.cib.tools import (
-    find_unique_id,
-    export_attributes,
-)
+from pcs.lib.cib.tools import find_unique_id
 from pcs.lib.errors import LibraryError
+from pcs.lib.xml_tools import export_attributes
 
 ATTRIB = {
     "sequential": ("true", "false"),
@@ -35,7 +33,7 @@ def validate_options(options):
     for name, value in options.items():
         if name not in ATTRIB:
             raise LibraryError(
-                reports.invalid_option(name, list(ATTRIB.keys()), None)
+                reports.invalid_option([name], list(ATTRIB.keys()), None)
             )
         if value not in ATTRIB[name]:
             raise LibraryError(
