@@ -35,7 +35,7 @@ from pcs.lib.external import (
 
 patch_commands = create_patcher("pcs.lib.commands.booth")
 
-@mock.patch("pcs.lib.booth.config_files.generate_key", return_value="key value")
+@mock.patch("pcs.lib.tools.generate_key", return_value="key value")
 @mock.patch("pcs.lib.commands.booth.build", return_value="config content")
 @mock.patch("pcs.lib.booth.config_structure.validate_peers")
 class ConfigSetupTest(TestCase):
@@ -111,6 +111,7 @@ class ConfigDestroyTest(TestCase):
     @patch_commands("parse", mock.Mock(side_effect=LibraryError()))
     def test_raises_when_cannot_get_content_of_config(self):
         env = mock.MagicMock()
+        env.booth.name = "somename"
         assert_raise_library_error(
             lambda: commands.config_destroy(env),
             (
@@ -126,6 +127,7 @@ class ConfigDestroyTest(TestCase):
     @patch_commands("parse", mock.Mock(side_effect=LibraryError()))
     def test_remove_config_even_if_cannot_get_its_content_when_forced(self):
         env = mock.MagicMock()
+        env.booth.name = "somename"
         env.report_processor = MockLibraryReportProcessor()
         commands.config_destroy(env, ignore_config_load_problems=True)
         env.booth.remove_config.assert_called_once_with()
@@ -544,7 +546,6 @@ class CreateInClusterTest(TestCase):
         assert_raise_library_error(
             lambda: commands.create_in_cluster(
                 mock.MagicMock(), "somename", ip="1.2.3.4",
-                resource_create=None, resource_remove=None,
             ),
             (
                 Severities.ERROR,
