@@ -5,14 +5,14 @@ from __future__ import (
     unicode_literals,
 )
 
-import os.path
-
 from lxml import etree
+import os.path
+import tempfile
 
 from pcs import settings
 from pcs.lib import reports
 from pcs.lib.booth.env import BoothEnv
-from pcs.lib.cib.tools import ensure_cib_version
+from pcs.lib.pacemaker.env import PacemakerEnv
 from pcs.lib.cluster_conf_facade import ClusterConfFacade
 from pcs.lib.corosync.config_facade import ConfigFacade as CorosyncConfigFacade
 from pcs.lib.corosync.live import (
@@ -33,12 +33,17 @@ from pcs.lib.nodes_task import (
     check_corosync_offline_on_nodes,
     qdevice_reload_on_nodes,
 )
-from pcs.lib.pacemaker import (
+from pcs.lib.pacemaker.live import (
+    ensure_wait_for_idle_support,
+    ensure_cib_version,
     get_cib,
     get_cib_xml,
     replace_cib_configuration_xml,
+    wait_for_idle,
+    get_cluster_status_xml,
 )
-
+from pcs.lib.pacemaker.state import get_cluster_state_dom
+from pcs.lib.pacemaker.values import get_valid_timeout_seconds
 
 class LibraryEnvironment(object):
     # pylint: disable=too-many-instance-attributes
@@ -52,8 +57,10 @@ class LibraryEnvironment(object):
         cib_data=None,
         corosync_conf_data=None,
         booth=None,
+        pacemaker=None,
         auth_tokens_getter=None,
         cluster_conf_data=None,
+        request_timeout=None,
     ):
         self._logger = logger
         self._report_processor = report_processor
@@ -65,6 +72,10 @@ class LibraryEnvironment(object):
         self._booth = (
             BoothEnv(report_processor, booth) if booth is not None else None
         )
+        #pacemaker is currently not mocked and it provides only an access to
+        #the authkey
+        self._pacemaker =  PacemakerEnv()
+        self._request_timeout = request_timeout
         self._is_cman_cluster = None
         # TODO tokens probably should not be inserted from outside, but we're
         # postponing dealing with them, because it's not that easy to move
@@ -72,6 +83,9 @@ class LibraryEnvironment(object):
         self._auth_tokens_getter = auth_tokens_getter
         self._auth_tokens = None
         self._cib_upgraded = False
+        self._cib_data_tmp_file = None
+
+        self.__timeout_cache = {}
 
     @property
     def logger(self):
@@ -109,32 +123,60 @@ class LibraryEnvironment(object):
         cib = get_cib(self._get_cib_xml())
         if minimal_version is not None:
             upgraded_cib = ensure_cib_version(
-                self.cmd_runner(), cib, minimal_version
+                self.cmd_runner(),
+                cib,
+                minimal_version
             )
             if upgraded_cib is not None:
                 cib = upgraded_cib
+                if self.is_cib_live and not self._cib_upgraded:
+                    self.report_processor.process(
+                        reports.cib_upgrade_successful()
+                    )
                 self._cib_upgraded = True
         return cib
 
+    def get_cluster_state(self):
+        return get_cluster_state_dom(get_cluster_status_xml(self.cmd_runner()))
+
     def _push_cib_xml(self, cib_data):
         if self.is_cib_live:
-            replace_cib_configuration_xml(
-                self.cmd_runner(), cib_data, self._cib_upgraded
-            )
-            if self._cib_upgraded:
-                self._cib_upgraded = False
-                self.report_processor.process(reports.cib_upgrade_successful())
+            replace_cib_configuration_xml(self.cmd_runner(), cib_data)
+            self._cib_upgraded = False
         else:
             self._cib_data = cib_data
 
+    def _get_wait_timeout(self, wait):
+        if wait is False:
+            return False
 
-    def push_cib(self, cib):
+        if wait not in self.__timeout_cache:
+            if not self.is_cib_live:
+                raise LibraryError(reports.wait_for_idle_not_live_cluster())
+            ensure_wait_for_idle_support(self.cmd_runner())
+            self.__timeout_cache[wait] = get_valid_timeout_seconds(wait)
+        return self.__timeout_cache[wait]
+
+
+    def ensure_wait_satisfiable(self, wait):
+        """
+        Raise when wait is not supported or when wait is not valid wait value.
+
+        mixed wait can be False when waiting is not required or valid timeout
+        """
+        self._get_wait_timeout(wait)
+
+    def push_cib(self, cib, wait=False):
+        timeout = self._get_wait_timeout(wait)
         #etree returns bytes: b'xml'
         #python 3 removed .encode() from bytes
         #run(...) calls subprocess.Popen.communicate which calls encode...
         #so here is bytes to str conversion
         self._push_cib_xml(etree.tostring(cib).decode())
 
+        if timeout is not False:
+            wait_for_idle(self.cmd_runner(), timeout)
+
     @property
     def is_cib_live(self):
         return self._cib_data is None
@@ -214,9 +256,9 @@ class LibraryEnvironment(object):
 
     def command_expect_live_corosync_env(self):
         if not self.is_corosync_conf_live:
-            raise LibraryError(reports.live_environment_required([
-                "--corosync_conf"
-            ]))
+            raise LibraryError(
+                reports.live_environment_required(["COROSYNC_CONF"])
+            )
 
     @property
     def is_corosync_conf_live(self):
@@ -227,8 +269,26 @@ class LibraryEnvironment(object):
             # make sure to get output of external processes in English and ASCII
             "LC_ALL": "C",
         }
+
         if self.user_login:
             runner_env["CIB_user"] = self.user_login
+
+        if not self.is_cib_live:
+            # Dump CIB data to a temporary file and set it up in the runner.
+            # This way every called pacemaker tool can access the CIB and we
+            # don't need to take care of it every time the runner is called.
+            if not self._cib_data_tmp_file:
+                try:
+                    self._cib_data_tmp_file = tempfile.NamedTemporaryFile(
+                        "w+",
+                        suffix=".pcs"
+                    )
+                    self._cib_data_tmp_file.write(self._get_cib_xml())
+                    self._cib_data_tmp_file.flush()
+                except EnvironmentError as e:
+                    raise LibraryError(reports.cib_save_tmp_error(str(e)))
+            runner_env["CIB_file"] = self._cib_data_tmp_file.name
+
         return CommandRunner(self.logger, self.report_processor, runner_env)
 
     def node_communicator(self):
@@ -237,7 +297,8 @@ class LibraryEnvironment(object):
             self.report_processor,
             self.__get_auth_tokens(),
             self.user_login,
-            self.user_groups
+            self.user_groups,
+            self._request_timeout
         )
 
     def __get_auth_tokens(self):
@@ -251,3 +312,7 @@ class LibraryEnvironment(object):
     @property
     def booth(self):
         return self._booth
+
+    @property
+    def pacemaker(self):
+        return self._pacemaker
