@@ -19,13 +19,15 @@ corosync_qdevice_net_client_certs_dir = os.path.join(
     "qdevice/net/nssdb"
 )
 corosync_qdevice_net_client_ca_file_name = "qnetd-cacert.crt"
+corosync_authkey_file = os.path.join(corosync_conf_dir, "authkey")
+pacemaker_authkey_file = "/etc/pacemaker/authkey"
 cluster_conf_file = "/etc/cluster/cluster.conf"
 fence_agent_binaries = "/usr/sbin/"
 pengine_binary = "/usr/libexec/pacemaker/pengine"
 crmd_binary = "/usr/libexec/pacemaker/crmd"
 cib_binary = "/usr/libexec/pacemaker/cib"
 stonithd_binary = "/usr/libexec/pacemaker/stonithd"
-pcs_version = "0.9.155"
+pcs_version = "0.9.159"
 crm_report = pacemaker_binaries + "crm_report"
 crm_verify = pacemaker_binaries + "crm_verify"
 crm_mon_schema = '/usr/share/pacemaker/crm_mon.rng'
@@ -39,8 +41,14 @@ pcsd_exec_location = "/usr/lib/pcsd/"
 cib_dir = "/var/lib/pacemaker/cib/"
 pacemaker_uname = "hacluster"
 pacemaker_gname = "haclient"
+sbd_binary = "/usr/sbin/sbd"
 sbd_watchdog_default = "/dev/watchdog"
 sbd_config = "/etc/sysconfig/sbd"
+# this limit is also mentioned in docs, change there as well
+sbd_max_device_num = 3
+# message types are also mentioned in docs, change there as well
+sbd_message_types = ["test", "reset", "off", "crashdump", "exit", "clear"]
 pacemaker_wait_timeout_status = 62
 booth_config_dir = "/etc/booth"
 booth_binary = "/usr/sbin/booth"
+default_request_timeout = 60
