@@ -8,6 +8,12 @@ from __future__ import (
 import sys
 import os.path
 
+is_2_7_or_higher = sys.version_info[0] > 2 or sys.version_info[1] > 6
+
+if is_2_7_or_higher:
+    import importlib
+
+
 PACKAGE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(
     os.path.abspath(__file__)
 )))
@@ -22,8 +28,24 @@ def prepare_test_name(test_name):
     but loader need it in module path format like:
     "pcs.test.test_node"
     so is practical accept fs path format and prepare it for loader
+
+    Sometimes name could include the .py extension:
+    "pcs/test/test_node.py"
+    in such cause is extension removed
     """
-    return test_name.replace("/", ".")
+    candidate = test_name.replace("/", ".")
+    if not is_2_7_or_higher:
+        return candidate
+
+    py_extension = ".py"
+    if not candidate.endswith(py_extension):
+        return candidate
+
+    try:
+        importlib.import_module(candidate)
+        return candidate
+    except ImportError:
+        return candidate[:-len(py_extension)]
 
 def tests_from_suite(test_candidate):
     if isinstance(test_candidate, unittest.TestCase):
@@ -60,15 +82,26 @@ def discover_tests(explicitly_enumerated_tests, exclude_enumerated_tests=False):
 explicitly_enumerated_tests = [
     prepare_test_name(arg) for arg in sys.argv[1:] if arg not in (
         "-v",
-        "--vanilla",
-        "--no-color", #deprecated, use --vanilla instead
         "--all-but",
+        "--fast-info", #show a traceback immediatelly after the test fails
         "--last-slash",
-        "--traditional-verbose",
+        "--list",
+        "--no-color", #deprecated, use --vanilla instead
         "--traceback-highlight",
+        "--traditional-verbose",
+        "--vanilla",
     )
 ]
 
+discovered_tests = discover_tests(
+    explicitly_enumerated_tests, "--all-but" in sys.argv
+)
+if "--list" in sys.argv:
+    test_list = tests_from_suite(discovered_tests)
+    print("\n".join(sorted(test_list)))
+    print("{0} tests found".format(len(test_list)))
+    sys.exit()
+
 if "--no-color" in sys.argv:
     print("DEPRECATED: --no-color is deprecated, use --vanilla instead")
 
@@ -90,15 +123,14 @@ if use_improved_result_class:
         slash_last_fail_in_overview=("--last-slash" in sys.argv),
         traditional_verbose=("--traditional-verbose" in sys.argv),
         traceback_highlight=("--traceback-highlight" in sys.argv),
+        fast_info=("--fast-info" in sys.argv)
     )
 
 testRunner = unittest.TextTestRunner(
     verbosity=2 if "-v" in sys.argv else 1,
     resultclass=resultclass
 )
-test_result =  testRunner.run(
-    discover_tests(explicitly_enumerated_tests, "--all-but" in sys.argv)
-)
+test_result =  testRunner.run(discovered_tests)
 if not test_result.wasSuccessful():
     sys.exit(1)
 
