@@ -8,8 +8,6 @@ from __future__ import (
 import os
 import sys
 import subprocess
-import ssl
-import inspect
 import xml.dom.minidom
 from xml.dom.minidom import parseString, parse
 import xml.etree.ElementTree as ET
@@ -25,19 +23,34 @@ import base64
 import threading
 import logging
 
-
 from pcs import settings, usage
-from pcs.cli.common.reports import (
-    process_library_reports,
-    LibraryReportProcessorToConsole as LibraryReportProcessorToConsole,
+
+from pcs.common import (
+    pcs_pycurl as pycurl,
+    report_codes,
 )
 from pcs.common.tools import (
     join_multilines,
     simple_cache,
 )
+
+from pcs.cli.common import (
+    console_report,
+    middleware,
+)
+from pcs.cli.common.env_cli import Env
+from pcs.cli.common.lib_wrapper import Library
+from pcs.cli.common.reports import (
+    build_report_message,
+    process_library_reports,
+    LibraryReportProcessorToConsole as LibraryReportProcessorToConsole,
+)
+from pcs.cli.booth.command import DEFAULT_BOOTH_NAME
+import pcs.cli.booth.env
+
 from pcs.lib import reports, sbd
 from pcs.lib.env import LibraryEnvironment
-from pcs.lib.errors import LibraryError
+from pcs.lib.errors import LibraryError, ReportListAnalyzer
 from pcs.lib.external import (
     CommandRunner,
     disable_service,
@@ -45,30 +58,24 @@ from pcs.lib.external import (
     enable_service,
     EnableServiceError,
     is_cman_cluster as lib_is_cman_cluster,
+    is_proxy_set,
     is_service_enabled,
     is_service_running,
     is_systemctl,
     _service,
     _systemctl,
 )
-import pcs.lib.resource_agent as lib_ra
 import pcs.lib.corosync.config_parser as corosync_conf_parser
 from pcs.lib.corosync.config_facade import ConfigFacade as corosync_conf_facade
-from pcs.lib.pacemaker import has_resource_wait_support
-from pcs.lib.pacemaker_state import ClusterState
-from pcs.lib.pacemaker_values import(
-    validate_id,
+from pcs.lib.nodes_task import check_can_add_node_to_cluster
+from pcs.lib.pacemaker.live import has_wait_for_idle_support
+from pcs.lib.pacemaker.state import ClusterState
+from pcs.lib.pacemaker.values import(
     is_boolean,
+    is_score as is_score_value,
     timeout_to_seconds as get_timeout_seconds,
-    is_score_value,
+    validate_id,
 )
-from pcs.cli.common import middleware
-from pcs.cli.common.env import Env
-from pcs.cli.common.lib_wrapper import Library
-from pcs.cli.common.reports import build_report_message
-from pcs.cli.booth.command import DEFAULT_BOOTH_NAME
-import pcs.cli.booth.env
-
 
 try:
     # python2
@@ -76,36 +83,10 @@ try:
 except ImportError:
     # python3
     from urllib.parse import urlencode as urllib_urlencode
-try:
-    # python2
-    from urllib2 import (
-        build_opener as urllib_build_opener,
-        install_opener as urllib_install_opener,
-        HTTPCookieProcessor as urllib_HTTPCookieProcessor,
-        HTTPSHandler as urllib_HTTPSHandler,
-        HTTPError as urllib_HTTPError,
-        URLError as urllib_URLError
-    )
-except ImportError:
-    # python3
-    from urllib.request import (
-        build_opener as urllib_build_opener,
-        install_opener as urllib_install_opener,
-        HTTPCookieProcessor as urllib_HTTPCookieProcessor,
-        HTTPSHandler as urllib_HTTPSHandler
-    )
-    from urllib.error import (
-        HTTPError as urllib_HTTPError,
-        URLError as urllib_URLError
-    )
-
-
 
 
 PYTHON2 = sys.version[0] == "2"
 
-DEFAULT_RESOURCE_ACTIONS = ["monitor", "start", "stop", "promote", "demote"]
-
 # usefile & filename variables are set in pcs module
 usefile = False
 filename = ""
@@ -146,6 +127,20 @@ def cluster_upgrade():
         err("unable to upgrade cluster: %s" % output)
     print("Cluster CIB has been upgraded to latest version")
 
+def cluster_upgrade_to_version(required_version):
+    checkAndUpgradeCIB(*required_version)
+    dom = get_cib_dom()
+    current_version = getValidateWithVersion(dom)
+    if current_version < required_version:
+        err(
+            console_report.CODE_TO_MESSAGE_BUILDER_MAP[
+                report_codes.CIB_UPGRADE_FAILED_TO_MINIMAL_REQUIRED_VERSION
+            ]({
+                "required_version": ".".join([str(x) for x in required_version]),
+                "current_version": ".".join([str(x) for x in current_version]),
+            })
+        )
+    return dom
 
 # Check status of node
 def checkStatus(node):
@@ -155,6 +150,11 @@ def checkStatus(node):
 def checkAuthorization(node):
     return sendHTTPRequest(node, 'remote/check_auth', None, False, False)
 
+def getPcsdInstanceSignature(node):
+    return sendHTTPRequest(
+        node, 'remote/pcsd_instance_signature', None, False, False
+    )
+
 def get_uid_gid_file_name(uid, gid):
     return "pcs-uidgid-%s-%s" % (uid, gid)
 
@@ -236,6 +236,23 @@ def readTokens():
         tokens = output['data']
     return tokens
 
+def repeat_if_timeout(send_http_request_function, repeat_count=15):
+    def repeater(node, *args, **kwargs):
+        repeats_left = repeat_count
+        while True:
+            retval, output = send_http_request_function(node, *args, **kwargs)
+            if (
+                retval != 2 or "Operation timed out" not in output
+                or
+                repeats_left < 1
+            ):
+                # did not timed out OR repeat limit exceeded
+                return retval, output
+            repeats_left = repeats_left - 1
+            if "--debug" in pcs_options:
+                print("{0}: {1}, trying again...". format(node, output))
+    return repeater
+
 # Set the corosync.conf file on the specified node
 def getCorosyncConfig(node):
     return sendHTTPRequest(node, 'remote/get_corosync_conf', None, False, False)
@@ -272,14 +289,23 @@ def stopCorosync(node, quiet=False, force=True):
 
 def stopCluster(node, quiet=False, pacemaker=True, corosync=True, force=True):
     data = dict()
+    timeout = None
     if pacemaker and not corosync:
         data["component"] = "pacemaker"
+        timeout = 2 * 60
     elif corosync and not pacemaker:
         data["component"] = "corosync"
     if force:
         data["force"] = 1
     data = urllib_urlencode(data)
-    return sendHTTPRequest(node, 'remote/cluster_stop', data, False, not quiet)
+    return sendHTTPRequest(
+        node,
+        'remote/cluster_stop',
+        data,
+        printResult=False,
+        printSuccess=not quiet,
+        timeout=timeout
+    )
 
 def enableCluster(node):
     return sendHTTPRequest(node, 'remote/cluster_enable', None, False, True)
@@ -302,24 +328,46 @@ def resumeConfigSyncing(node):
     data = urllib_urlencode({"sync_thread_resume": 1})
     return sendHTTPRequest(node, "remote/set_sync_options", data, False, False)
 
-def canAddNodeToCluster(node):
-    retval, output = sendHTTPRequest(
-        node, 'remote/node_available', None, False, False
+def canAddNodeToCluster(node_communicator, node):
+    """
+    Return tuple with two parts. The first part is information if the node can
+    be added to a cluster. The second part is a relevant explanation for first
+    part.
+
+    NodeCommunicator node_communicator provide connection to the node
+    NodeAddresses node contain destination for request
+    """
+    report_list = []
+    check_can_add_node_to_cluster(node_communicator, node, report_list)
+
+    analyzer = ReportListAnalyzer(report_list)
+    if not analyzer.error_list:
+        return True, ""
+
+    first_problem = analyzer.error_list[0]
+
+    report_message_map = {
+        report_codes.NODE_COMMUNICATION_ERROR_NOT_AUTHORIZED:
+           "unable to authenticate to node"
+        ,
+        report_codes.CANNOT_ADD_NODE_IS_IN_CLUSTER:
+            "node is already in a cluster"
+        ,
+        report_codes.INVALID_RESPONSE_FORMAT:
+            "response parsing error"
+        ,
+        report_codes.CANNOT_ADD_NODE_IS_RUNNING_SERVICE:
+             "node is running pacemaker_remote"
+        ,
+    }
+
+    if first_problem.code in report_message_map:
+        return False, report_message_map[first_problem.code]
+
+    return False, "error checking node availability{0}".format(
+        ": {0}".format(first_problem.info["reason"])
+            if "reason" in first_problem.info else ""
     )
-    if retval == 0:
-        try:
-            myout = json.loads(output)
-            if "notauthorized" in myout and myout["notauthorized"] == "true":
-                return (False, "unable to authenticate to node")
-            if "node_available" in myout and myout["node_available"] == True:
-                return (True, "")
-            elif myout.get("pacemaker_remote", False):
-                return (False, "node is running pacemaker_remote")
-            else:
-                return (False, "node is already in a cluster")
-        except ValueError:
-            return (False, "response parsing error")
-    return (False, "error checking node availability: {0}".format(output))
 
 def addLocalNode(node, node_to_add, ring1_addr=None):
     options = {'new_nodename': node_to_add}
@@ -350,6 +398,7 @@ def removeLocalNode(node, node_to_remove, pacemaker_remove=False):
     else:
         return 1, output
 
+
 # Send an HTTP request to a node return a tuple with status, data
 # If status is 0 then data contains server response
 # Otherwise if non-zero then data contains error message
@@ -359,32 +408,113 @@ def removeLocalNode(node, node_to_remove, pacemaker_remove=False):
 # 2 = No response,
 # 3 = Auth Error
 # 4 = Permission denied
-def sendHTTPRequest(host, request, data = None, printResult = True, printSuccess = True):
-    url = 'https://' + host + ':2224/' + request
-    # enable self-signed certificates
-    # https://www.python.org/dev/peps/pep-0476/
-    # http://bugs.python.org/issue21308
-    if (
-        hasattr(ssl, "_create_unverified_context")
-        and
-        "context" in inspect.getargspec(urllib_HTTPSHandler.__init__).args
-    ):
-        opener = urllib_build_opener(
-            urllib_HTTPSHandler(context=ssl._create_unverified_context()),
-            urllib_HTTPCookieProcessor()
-        )
-    else:
-        opener = urllib_build_opener(urllib_HTTPCookieProcessor())
-
-    tokens = readTokens()
+def sendHTTPRequest(
+    host, request, data=None, printResult=True, printSuccess=True, timeout=None
+):
+    url = "https://{host}:2224/{request}".format(host=host, request=request)
     if "--debug" in pcs_options:
         print("Sending HTTP Request to: " + url)
         print("Data: {0}".format(data))
-    # python3 requires data to by bytes not str
-    if data:
-        data = data.encode("utf-8")
 
-    # cookies
+    def __debug_callback(data_type, debug_data):
+        prefixes = {
+            pycurl.DEBUG_TEXT: b"* ",
+            pycurl.DEBUG_HEADER_IN: b"< ",
+            pycurl.DEBUG_HEADER_OUT: b"> ",
+            pycurl.DEBUG_DATA_IN: b"<< ",
+            pycurl.DEBUG_DATA_OUT: b">> ",
+        }
+        if data_type in prefixes:
+            debug_output.write(prefixes[data_type])
+            debug_output.write(debug_data)
+            if not debug_data.endswith(b"\n"):
+                debug_output.write(b"\n")
+
+    output = BytesIO()
+    debug_output = BytesIO()
+    cookies = __get_cookie_list(host, readTokens())
+    if not timeout:
+        timeout = settings.default_request_timeout
+    if "--request-timeout" in pcs_options:
+        timeout = pcs_options["--request-timeout"]
+
+    handler = pycurl.Curl()
+    handler.setopt(pycurl.PROTOCOLS, pycurl.PROTO_HTTPS)
+    handler.setopt(pycurl.URL, url.encode("utf-8"))
+    handler.setopt(pycurl.WRITEFUNCTION, output.write)
+    handler.setopt(pycurl.VERBOSE, 1)
+    handler.setopt(pycurl.NOSIGNAL, 1) # required for multi-threading
+    handler.setopt(pycurl.DEBUGFUNCTION, __debug_callback)
+    handler.setopt(pycurl.TIMEOUT_MS, int(timeout * 1000))
+    handler.setopt(pycurl.SSL_VERIFYHOST, 0)
+    handler.setopt(pycurl.SSL_VERIFYPEER, 0)
+    if cookies:
+        handler.setopt(pycurl.COOKIE, ";".join(cookies).encode("utf-8"))
+    if data:
+        handler.setopt(pycurl.COPYPOSTFIELDS, data.encode("utf-8"))
+    try:
+        handler.perform()
+        response_data = output.getvalue().decode("utf-8")
+        response_code = handler.getinfo(pycurl.RESPONSE_CODE)
+        if printResult or printSuccess:
+            print(host + ": " + response_data.strip())
+        if "--debug" in pcs_options:
+            print("Response Code: {0}".format(response_code))
+            print("--Debug Response Start--\n{0}".format(response_data))
+            print("--Debug Response End--")
+            print("Communication debug info for calling: {0}".format(url))
+            print("--Debug Communication Output Start--")
+            print(debug_output.getvalue().decode("utf-8", "ignore"))
+            print("--Debug Communication Output End--")
+            print()
+
+        if response_code == 401:
+            output = (
+                3,
+                (
+                    "Unable to authenticate to {node} - (HTTP error: {code}), "
+                    "try running 'pcs cluster auth'"
+                ).format(node=host, code=response_code)
+            )
+        elif response_code == 403:
+            output = (
+                4,
+                "{node}: Permission denied - (HTTP error: {code})".format(
+                    node=host, code=response_code
+                )
+            )
+        elif response_code >= 400:
+            output = (
+                1,
+                "Error connecting to {node} - (HTTP error: {code})".format(
+                    node=host, code=response_code
+                )
+            )
+        else:
+            output = (0, response_data)
+
+        if printResult and output[0] != 0:
+            print(output[1])
+
+        return output
+    except pycurl.error as e:
+        if is_proxy_set(os.environ):
+            print(
+                "Warning: Proxy is set in environment variables, try "
+                "disabling it"
+            )
+        dummy_errno, reason = e.args
+        if "--debug" in pcs_options:
+            print("Response Reason: {0}".format(reason))
+        msg = "Unable to connect to {host} ({reason})".format(
+            host=host, reason=reason
+        )
+        if printResult:
+            print(msg)
+        return (2, msg)
+
+
+def __get_cookie_list(host, tokens):
     cookies = []
     if host in tokens:
         cookies.append("token=" + tokens[host])
@@ -399,61 +529,12 @@ def sendHTTPRequest(host, request, data = None, printResult = True, printSuccess
                     value = re.sub(r"[^!-~]", "", value).replace(";", "")
                 else:
                     # python3 requires the value to be bytes not str
-                    value = base64.b64encode(value.encode("utf8"))
+                    value = base64.b64encode(
+                        value.encode("utf8")
+                    ).decode("utf-8")
                 cookies.append("{0}={1}".format(name, value))
-    if cookies:
-        opener.addheaders.append(('Cookie', ";".join(cookies)))
+    return cookies
 
-    # send the request
-    urllib_install_opener(opener)
-    try:
-        result = opener.open(url,data)
-        # python3 returns bytes not str
-        html = result.read().decode("utf-8")
-        if printResult or printSuccess:
-            print(host + ": " + html.strip())
-        if "--debug" in pcs_options:
-            print("Response Code: 0")
-            print("--Debug Response Start--\n{0}".format(html), end="")
-            print("--Debug Response End--")
-            print()
-        return (0,html)
-    except urllib_HTTPError as e:
-        if "--debug" in pcs_options:
-            print("Response Code: " + str(e.code))
-            html = e.read().decode("utf-8")
-            print("--Debug Response Start--\n{0}".format(html), end="")
-            print("--Debug Response End--")
-        if e.code == 401:
-            output = (
-                3,
-                "Unable to authenticate to {node} - (HTTP error: {code}), try running 'pcs cluster auth'".format(
-                    node=host, code=e.code
-                )
-            )
-        elif e.code == 403:
-            output = (
-                4,
-                "{node}: Permission denied - (HTTP error: {code})".format(
-                    node=host, code=e.code
-                )
-            )
-        else:
-            output = (
-                1,
-                "Error connecting to {node} - (HTTP error: {code})".format(
-                    node=host, code=e.code
-                )
-            )
-        if printResult:
-            print(output[1])
-        return output
-    except urllib_URLError as e:
-        if "--debug" in pcs_options:
-            print("Response Reason: " + str(e.reason))
-        if printResult:
-            print("Unable to connect to %s (%s)" % (host, e.reason))
-        return (2,"Unable to connect to %s (%s)" % (host, e.reason))
 
 def getNodesFromCorosyncConf(conf_text=None):
     if is_rhel6():
@@ -950,6 +1031,10 @@ def run_pcsdcli(command, data=None):
     env_var = dict()
     if "--debug" in pcs_options:
         env_var["PCSD_DEBUG"] = "true"
+    if "--request-timeout" in pcs_options:
+        env_var["PCSD_NETWORK_TIMEOUT"] = str(pcs_options["--request-timeout"])
+    else:
+        env_var["PCSD_NETWORK_TIMEOUT"] = str(settings.default_request_timeout)
     pcs_dir = os.path.realpath(os.path.dirname(sys.argv[0]))
     if pcs_dir == "/usr/sbin":
         pcsd_dir_path = settings.pcsd_exec_location
@@ -958,20 +1043,33 @@ def run_pcsdcli(command, data=None):
     pcsdcli_path = os.path.join(pcsd_dir_path, 'pcsd-cli.rb')
     gem_home = os.path.join(pcsd_dir_path, 'vendor/bundle/ruby')
     env_var["GEM_HOME"] = gem_home
-    output, retval = run(
+    stdout, dummy_stderr, retval = cmd_runner().run(
         ["/usr/bin/ruby", "-I" + pcsd_dir_path, pcsdcli_path, command],
-        string_for_stdin=json.dumps(data),
-        env_extend=env_var
+        json.dumps(data),
+        env_var
     )
     try:
-        output_json = json.loads(output)
+        output_json = json.loads(stdout)
         for key in ['status', 'text', 'data']:
             if key not in output_json:
                 output_json[key] = None
+
+        output = "".join(output_json['log'])
+        # check if some requests timed out, if so print message about it
+        if "error: operation_timedout" in output:
+            print("Error: Operation timed out")
+        # check if there are any connection failures due to proxy in pcsd and
+        # print warning if so
+        proxy_msg = (
+            'Proxy is set in environment variables, try disabling it'
+        )
+        if proxy_msg in output:
+            print("Warning: {0}".format(proxy_msg))
+
     except ValueError:
         output_json = {
             'status': 'bad_json_output',
-            'text': output,
+            'text': stdout,
             'data': None,
         }
     return output_json, retval
@@ -1196,16 +1294,18 @@ def dom_get_resource_clone_ms_parent(dom, resource_id):
         or
         dom_get_group(dom, resource_id)
     )
-    return dom_elem_get_resource_clone_ms_parent(resource)
+    if resource:
+        return dom_elem_get_resource_clone_ms_parent(resource)
+    return None
 
 def dom_elem_get_resource_clone_ms_parent(resource):
-    clone = resource
-    while True:
-        if not isinstance(clone, xml.dom.minidom.Element):
-            return None
-        if clone.tagName in ["clone", "master"]:
-            return clone
-        clone = clone.parentNode
+    return dom_get_parent_by_tag_names(resource, ["clone", "master"])
+
+def dom_get_resource_bundle_parent(dom, resource_id):
+    resource = dom_get_resource(dom, resource_id)
+    if resource:
+        return dom_get_parent_by_tag_names(resource, ["bundle"])
+    return None
 
 def dom_get_master(dom, master_id):
     for master in dom.getElementsByTagName("master"):
@@ -1225,6 +1325,22 @@ def dom_get_group(dom, group_id):
             return group
     return None
 
+def dom_get_bundle(dom, bundle_id):
+    for bundle in dom.getElementsByTagName("bundle"):
+        if bundle.getAttribute("id") == bundle_id:
+            return bundle
+    return None
+
+def dom_get_resource_bundle(bundle_el):
+    for child in bundle_el.childNodes:
+        if (
+            child.nodeType == xml.dom.minidom.Node.ELEMENT_NODE
+            and
+            child.tagName == "primitive"
+        ):
+            return child
+    return None
+
 def dom_get_group_clone(dom, group_id):
     for clone in dom.getElementsByTagName("clone"):
         group = dom_get_group(clone, group_id)
@@ -1281,9 +1397,11 @@ def validate_constraint_resource(dom, resource_id):
         dom_get_clone(dom, resource_id)
         or
         dom_get_master(dom, resource_id)
+        or
+        dom_get_bundle(dom, resource_id)
     )
     if resource_el:
-        # clone and master is always valid
+        # clones, masters and bundles are always valid
         return True, "", resource_id
 
     resource_el = (
@@ -1294,9 +1412,14 @@ def validate_constraint_resource(dom, resource_id):
     if not resource_el:
         return False, "Resource '%s' does not exist" % resource_id, None
 
-    clone_el = dom_get_resource_clone_ms_parent(dom, resource_id)
+    clone_el = (
+        dom_get_resource_clone_ms_parent(dom, resource_id)
+        or
+        dom_get_resource_bundle_parent(dom, resource_id)
+    )
     if not clone_el:
-        # primitive and group is valid if not in clone nor master
+        # a primitive and a group is valid if not in a clone nor a master nor a
+        # bundle
         return True, "", resource_id
 
     if "--force" in pcs_options:
@@ -1322,6 +1445,14 @@ def validate_constraint_resource(dom, resource_id):
                 % (resource_id, clone_el.getAttribute("id")),
             clone_el.getAttribute("id")
         )
+    if clone_el.tagName == "bundle":
+        return (
+            False,
+            "%s is a bundle resource, you should use the bundle id: %s "
+                "when adding constraints. Use --force to override."
+                % (resource_id, clone_el.getAttribute("id")),
+            clone_el.getAttribute("id")
+        )
     return True, "", resource_id
 
 
@@ -1371,12 +1502,12 @@ def dom_get_child_by_tag_name(dom_el, tag_name):
         return children[0]
     return None
 
-def dom_get_parent_by_tag_name(dom_el, tag_name):
+def dom_get_parent_by_tag_names(dom_el, tag_names):
     parent = dom_el.parentNode
     while parent:
         if not isinstance(parent, xml.dom.minidom.Element):
             return None
-        if parent.tagName == tag_name:
+        if parent.tagName in tag_names:
             return parent
         parent = parent.parentNode
     return None
@@ -1390,6 +1521,7 @@ def dom_attrs_to_list(dom_el, with_id=False):
         attributes.append("(id:%s)" % (dom_el.getAttribute("id")))
     return attributes
 
+# moved to pcs.lib.pacemaker.state
 def get_resource_for_running_check(cluster_state, resource_id, stopped=False):
     for clone in cluster_state.getElementsByTagName("clone"):
         if clone.getAttribute("id") == resource_id:
@@ -1420,6 +1552,8 @@ def get_resource_for_running_check(cluster_state, resource_id, stopped=False):
             resource_id = elem.getAttribute("id")
     return resource_id
 
+# moved to pcs.lib.pacemaker.state
+# see pcs.lib.commands.resource for usage
 def resource_running_on(resource, passed_state=None, stopped=False):
     nodes_started = []
     nodes_master = []
@@ -1477,56 +1611,15 @@ def resource_running_on(resource, passed_state=None, stopped=False):
         "nodes_slave": nodes_slave,
     }
 
-def filter_default_op_from_actions(resource_actions):
-    filtered = []
-    for action in resource_actions:
-        if action.get("name", "") not in DEFAULT_RESOURCE_ACTIONS:
-            continue
-        new_action = dict([
-            (name, value)
-            for name, value in action.items()
-            if name != "depth"
-        ])
-        filtered.append(new_action)
-    return filtered
-
-# Given a resource agent (ocf:heartbeat:XXX) return an list of default
-# operations or an empty list if unable to find any default operations
-def get_default_op_values(full_agent_name):
-    default_ops = []
-    try:
-        if full_agent_name.startswith("stonith:"):
-            metadata = lib_ra.StonithAgent(
-                cmd_runner(),
-                full_agent_name[len("stonith:"):]
-            )
-        else:
-            metadata = lib_ra.ResourceAgent(
-                cmd_runner(),
-                full_agent_name
-            )
-        actions = filter_default_op_from_actions(metadata.get_actions())
-
-        for action in actions:
-            op = [action["name"]]
-            for key in action.keys():
-                if key != "name" and action[key] != "0":
-                    op.append("{0}={1}".format(key, action[key]))
-            default_ops.append(op)
-    except lib_ra.UnableToGetAgentMetadata:
-        return []
-    except lib_ra.ResourceAgentError as e:
-        process_library_reports(
-            [lib_ra.resource_agent_error_to_report_item(e)]
-        )
-    except LibraryError as e:
-        process_library_reports(e.args)
-
-    return default_ops
-
+def agent_action_to_cmdline_format(action):
+    op = [action["name"]]
+    for key in action.keys():
+        if key != "name" and action[key] != "0":
+            op.append("{0}={1}".format(key, action[key]))
+    return op
 
 def check_pacemaker_supports_resource_wait():
-    if not has_resource_wait_support(cmd_runner()):
+    if not has_wait_for_idle_support(cmd_runner()):
         err("crm_resource does not support --wait, please upgrade pacemaker")
 
 def validate_wait_get_timeout(need_cib_support=True):
@@ -1850,48 +1943,13 @@ def get_terminal_password(message="Password: "):
 def getClusterState():
     return parseString(getClusterStateXml())
 
-# DEPRECATED, please use lib.pacemaker.get_cluster_status_xml in new code
+# DEPRECATED, please use lib.pacemaker.live.get_cluster_status_xml in new code
 def getClusterStateXml():
     xml, returncode = run(["crm_mon", "--one-shot", "--as-xml", "--inactive"])
     if returncode != 0:
         err("error running crm_mon, is pacemaker running?")
     return xml
 
-# Returns true if stonith-enabled is not false/off & no stonith devices exist
-# So if the cluster can't start due to missing stonith devices return true
-def stonithCheck():
-    et = get_cib_etree()
-    cps = et.find("configuration/crm_config/cluster_property_set")
-    if cps != None:
-        for prop in cps.findall(str("nvpair")):
-            if 'name' in prop.attrib and prop.attrib["name"] == "stonith-enabled":
-                if prop.attrib["value"] == "off" or \
-                        prop.attrib["value"] == "false":
-                    return False
-
-    xpath_list = (
-        "configuration/resources/primitive",
-        "configuration/resources/group/primitive",
-        "configuration/resources/clone/primitive",
-        "configuration/resources/clone/group/primitive",
-        "configuration/resources/master/primitive",
-        "configuration/resources/master/group/primitive",
-    )
-    for xpath in xpath_list:
-        for p in et.findall(str(xpath)):
-            if ("class" in p.attrib) and (p.attrib["class"] == "stonith"):
-                return False
-
-    if not usefile:
-        # check if SBD daemon is running
-        try:
-            if is_service_running(cmd_runner(), sbd.get_sbd_service_name()):
-                return False
-        except LibraryError:
-            pass
-
-    return True
-
 def getCorosyncNodesID(allow_failure=False):
     if os.getuid() == 0:
         if is_rhel6():
@@ -2724,6 +2782,7 @@ def get_lib_env():
         cib_data,
         corosync_conf_data,
         auth_tokens_getter=readTokens,
+        request_timeout=pcs_options.get("--request-timeout"),
     )
 
 def get_cli_env():
@@ -2743,6 +2802,7 @@ def get_cli_env():
     env.groups = groups
     env.auth_tokens_getter = readTokens
     env.debug = "--debug" in pcs_options
+    env.request_timeout = pcs_options.get("--request-timeout")
     return env
 
 def get_middleware_factory():
@@ -2773,16 +2833,27 @@ def get_modificators():
     #there is possible create class extending dict, so dict like access in
     #commands is not an issue
     return {
+        "after": pcs_options.get("--after", None),
+        "all": "--all" in pcs_options,
+        "async": "--async" in pcs_options,
         "autocorrect": "--autocorrect" in pcs_options,
         "autodelete": "--autodelete" in pcs_options,
+        "before": pcs_options.get("--before", None),
         "corosync_conf": pcs_options.get("--corosync_conf", None),
         "describe": "--nodesc" not in pcs_options,
+        "device": pcs_options.get("--device", []),
+        "disabled": "--disabled" in pcs_options,
         "enable": "--enable" in pcs_options,
+        "encryption": pcs_options.get("--encryption", "0"),
         "force": "--force" in pcs_options,
         "full": "--full" in pcs_options,
+        "group": pcs_options.get("--group", None),
+        "monitor": "--monitor" in pcs_options,
         "name": pcs_options.get("--name", None),
+        "no-default-ops": "--no-default-ops" in pcs_options,
         "skip_offline_nodes": "--skip-offline" in pcs_options,
         "start": "--start" in pcs_options,
+        "wait": pcs_options.get("--wait", False),
         "watchdog": pcs_options.get("--watchdog", []),
     }
 
