@@ -1,14 +1,17 @@
 # Wrapper for PCS command
-#
+
+require 'etc'
 require 'open4'
 require 'shellwords'
 require 'cgi'
 require 'net/http'
 require 'net/https'
+require 'uri'
 require 'json'
 require 'fileutils'
 require 'backports'
 require 'base64'
+require 'ethon'
 
 require 'config.rb'
 require 'cfgsync.rb'
@@ -58,9 +61,16 @@ def add_node_attr(auth_user, node, key, value)
 end
 
 def add_meta_attr(auth_user, resource, key, value)
-  stdout, stderr, retval = run_cmd(
-    auth_user, PCS, "resource", "meta", resource, key.to_s + "=" + value.to_s
-  )
+  cmd = ["resource", "meta", resource, key.to_s + "=" + value.to_s]
+  if key.to_s == "remote-node"
+    # --force is a workaround for:
+    # 1) Error: this command is not sufficient for create guest node, use 'pcs
+    # cluster node add-guest', use --force to override
+    # 2) Error: this command is not sufficient for remove guest node, use 'pcs
+    # cluster node remove-guest', use --force to override
+    cmd << "--force"
+  end
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *cmd)
   return retval
 end
 
@@ -391,7 +401,7 @@ def send_nodes_request_with_token(auth_user, nodes, request, post=false, data={}
   return code, out
 end
 
-def send_request_with_token(auth_user, node, request, post=false, data={}, remote=true, raw_data=nil, timeout=30, additional_tokens={})
+def send_request_with_token(auth_user, node, request, post=false, data={}, remote=true, raw_data=nil, timeout=nil, additional_tokens={})
   token = additional_tokens[node] || get_node_token(node)
   $logger.info "SRWT Node: #{node} Request: #{request}"
   if not token
@@ -406,31 +416,8 @@ def send_request_with_token(auth_user, node, request, post=false, data={}, remot
   )
 end
 
-def send_request(auth_user, node, request, post=false, data={}, remote=true, raw_data=nil, timeout=30, cookies_data=nil)
-  cookies_data = {} if not cookies_data
-  request = "/#{request}" if not request.start_with?("/")
-
-  # fix ipv6 address for URI.parse
-  node6 = node
-  if (node.include?(":") and ! node.start_with?("["))
-    node6 = "[#{node}]"
-  end
-
-  if remote
-    uri = URI.parse("https://#{node6}:2224/remote" + request)
-  else
-    uri = URI.parse("https://#{node6}:2224" + request)
-  end
-
-  if post
-    req = Net::HTTP::Post.new(uri.path)
-    raw_data ? req.body = raw_data : req.set_form_data(data)
-  else
-    req = Net::HTTP::Get.new(uri.path)
-    req.set_form_data(data)
-  end
-
-  cookies_to_send = []
+def _get_cookie_list(auth_user, cookies_data)
+  cookie_list = []
   cookies_data_default = {}
   # Let's be safe about characters in cookie variables and do base64.
   # We cannot do it for CIB_user however to be backward compatible
@@ -444,36 +431,123 @@ def send_request(auth_user, node, request, post=false, data={}, remote=true, raw
 
   cookies_data_default.update(cookies_data)
   cookies_data_default.each { |name, value|
-    cookies_to_send << CGI::Cookie.new('name' => name, 'value' => value).to_s
+    cookie_list << CGI::Cookie.new('name' => name, 'value' => value).to_s
   }
-  req.add_field('Cookie', cookies_to_send.join(';'))
+  return cookie_list
+end
 
-  begin
-    # uri.host returns "[addr]" for ipv6 addresses, which is wrong
-    # uri.hostname returns "addr" for ipv6 addresses, which is correct, but it
-    #   is not available in older ruby versions
-    # There is a bug in Net::HTTP.new in some versions of ruby which prevents
-    # ipv6 addresses being used here at all.
-    myhttp = Net::HTTP.new(node, uri.port)
-    myhttp.use_ssl = true
-    myhttp.verify_mode = OpenSSL::SSL::VERIFY_NONE
-    res = myhttp.start do |http|
-      http.read_timeout = timeout
-      http.request(req)
+def _transform_data(data)
+  # Converts data in a way that URI.encode_www_form method will encode it
+  # corectly. If an arrray is passed as value to encode_www_form, then parser of
+  # webbrick will use only last value.
+  new_data = []
+  data.each { |key, val|
+    if val.kind_of?(Array)
+      val.each { |value|
+        new_data << ["#{key.to_s}[]", value]
+      }
+    else
+      new_data << [key, val]
     end
-    return res.code.to_i, res.body
-  rescue Exception => e
-    $logger.info "No response from: #{node} request: #{request}, exception: #{e}"
+  }
+  return new_data
+end
+
+def send_request(
+  auth_user, node, request, post=false, data={}, remote=true, raw_data=nil,
+  timeout=nil, cookies_data=nil
+)
+  cookies_data = {} if not cookies_data
+  if request.start_with?("/")
+    request.slice!(0)
+  end
+
+  node6 = node
+  if (node.include?(":") and ! node.start_with?("["))
+    node6 = "[#{node}]"
+  end
+
+  if remote
+    url = "https://#{node6}:2224/remote/#{request}"
+  else
+    url = "https://#{node6}:2224/#{request}"
+  end
+
+  data = _transform_data(data)
+
+  if post
+    encoded_data = (raw_data) ? raw_data : URI.encode_www_form(data)
+  else
+    url_data = (raw_data) ? raw_data : URI.encode_www_form(data)
+    prefix = request.include?('?') ? '&' : '?'
+    url += "#{prefix}#{url_data}"
+  end
+
+
+  timeout_ms = 30000
+  begin
+    if timeout
+      timeout_ms = (Float(timeout) * 1000).to_i
+    elsif ENV['PCSD_NETWORK_TIMEOUT']
+       timeout_ms = (Float(ENV['PCSD_NETWORK_TIMEOUT']) * 1000).to_i
+    end
+  rescue
+  end
+
+  req = Ethon::Easy.new()
+  req.set_attributes({
+    :url => url,
+    :timeout_ms => timeout_ms,
+    :cookie => _get_cookie_list(auth_user, cookies_data).join(';'),
+    :ssl_verifyhost => 0,
+    :ssl_verifypeer => 0,
+    :postfields => (encoded_data) ? encoded_data : nil,
+    :httpget => (post ? 0 : 1),
+    :nosignal => 1, # required for multi-threading
+  })
+  return_code = req.perform
+  if return_code == :ok
+    return req.response_code, req.response_body
+  else
+    if is_proxy_set(ENV)
+      $logger.warn(
+        'Proxy is set in environment variables, try disabling it'
+      )
+    end
+    $logger.info(
+      "No response from: #{node} request: #{request}, error: #{return_code}"
+    )
     return 400,'{"noresponse":true}'
   end
 end
 
-def add_node(auth_user, new_nodename, all=false, auto_start=true, watchdog=nil)
+def is_proxy_set(env_var_hash)
+  proxy_list = ["https_proxy", "all_proxy"]
+  proxy_list += proxy_list.map {|item| item.upcase}
+  proxy_list.each { |var|
+    if env_var_hash[var] and env_var_hash[var] != ''
+      return true
+    end
+  }
+  return false
+end
+
+def add_node(
+  auth_user, new_nodename, all=false, auto_start=true, watchdog=nil,
+  device_list=nil
+)
   if all
     command = [PCS, "cluster", "node", "add", new_nodename]
     if watchdog and not watchdog.strip.empty?
       command << "--watchdog=#{watchdog.strip}"
     end
+    if device_list
+      device_list.each { |device|
+        if device and not device.strip.empty?
+          command << "--device=#{device.strip}"
+        end
+      }
+    end
     if auto_start
       command << '--start'
       command << '--enable'
@@ -686,16 +760,16 @@ def get_resource_agents_avail(auth_user, params)
   code, result = send_cluster_request_with_token(
     auth_user, params[:cluster], 'get_avail_resource_agents'
   )
-  return {} if 200 != code
+  return [] if 200 != code
   begin
     ra = JSON.parse(result)
     if (ra["noresponse"] == true) or (ra["notauthorized"] == "true") or (ra["notoken"] == true) or (ra["pacemaker_not_running"] == true)
-      return {}
+      return []
     else
-      return ra
+      return ra.keys
     end
   rescue JSON::ParserError
-    return {}
+    return []
   end
 end
 
@@ -815,11 +889,13 @@ def get_fence_levels(auth_user, cib_dom=nil)
     '/cib/configuration/fencing-topology/fencing-level'
   ) { |e|
     target = e.attributes['target']
-    fence_levels[target] ||= []
-    fence_levels[target] << {
-      'level' => e.attributes['index'],
-      'devices' => e.attributes['devices']
-    }
+    if target
+      fence_levels[target] ||= []
+      fence_levels[target] << {
+        'level' => e.attributes['index'],
+        'devices' => e.attributes['devices']
+      }
+    end
   }
 
   fence_levels.each { |_, val| val.sort_by! { |obj| obj['level'].to_i }}
@@ -955,7 +1031,11 @@ def get_rhel_version()
 end
 
 def pcsd_restart()
+  # restart in a separate process so we can send a response to the restart
+  # request
   fork {
+    # let us send the response to the restart request
+    # we need little bit more time to finish some things when setting up cluster
     sleep(10)
     if ISSYSTEMCTL
       exec("systemctl", "restart", "pcsd")
@@ -1337,10 +1417,18 @@ def pcsd_restart_nodes(auth_user, nodes)
   node_status = {}
   node_response.each { |node, response|
     if response[0] == 200
-      node_status[node] = {
+      my_status = {
         'status' => 'ok',
         'text' => 'Success',
       }
+      begin
+        parsed_response = JSON.parse(response[1], {:symbolize_names => true})
+        if parsed_response[:instance_signature]
+          my_status["instance_signature"] = parsed_response[:instance_signature]
+        end
+      rescue JSON::ParserError
+      end
+      node_status[node] = my_status
     else
       text = response[1]
       if response[0] == 401
@@ -1371,11 +1459,22 @@ def pcsd_restart_nodes(auth_user, nodes)
   }
 end
 
-def write_file_lock(path, perm, data, binary=false)
+def get_uid(username)
+  return Etc.getpwnam(username).uid
+end
+
+def get_gid(groupname)
+  return Etc.getgrnam(groupname).gid
+end
+
+def write_file_lock(path, perm, data, binary=false, user=nil, group=nil)
   file = nil
   begin
     file = File.open(path, binary ? 'wb' : 'w', perm)
     file.flock(File::LOCK_EX)
+    if user or group
+      File.chown(get_uid(user), get_gid(group), path)
+    end
     file.write(data)
   rescue => e
     $logger.error("Cannot save file '#{path}': #{e.message}")
@@ -1737,6 +1836,7 @@ def get_node_status(auth_user, cib_dom)
         'moving_resource_in_group',
         'unmanaged_resource',
         'alerts',
+        'hardened_cluster',
       ]
   }
 
@@ -1999,6 +2099,9 @@ def is_service_installed?(service)
     return false
   end
 
+  # currently we are not using systemd instances (service_name@instance) in pcsd
+  # for proper implementation of is_service_installed see
+  # pcs/lib/external.py:is_service_installed
   stdout, _, retcode = run_cmd(
     PCSAuth.getSuperuserAuth(), 'systemctl', 'list-unit-files', '--full'
   )
@@ -2095,13 +2198,6 @@ def get_sbd_service_name()
   end
 end
 
-def write_booth_config(config, data)
-  if config.include?('/')
-    raise InvalidFileNameException.new(config)
-  end
-  write_file_lock(File.join(BOOTH_CONFIG_DIR, config), nil, data)
-end
-
 def read_booth_config(config)
   if config.include?('/')
     raise InvalidFileNameException.new(config)
@@ -2113,15 +2209,6 @@ def read_booth_config(config)
   return read_file_lock(config_path)
 end
 
-def write_booth_authfile(filename, data)
-  if filename.include?('/')
-    raise InvalidFileNameException.new(filename)
-  end
-  write_file_lock(
-    File.join(BOOTH_CONFIG_DIR, filename), 0600, Base64.decode64(data), true
-  )
-end
-
 def read_booth_authfile(filename)
   if filename.include?('/')
     raise InvalidFileNameException.new(filename)
