@@ -107,7 +107,7 @@ configure do
   DISABLE_GUI = (
     ENV['PCSD_DISABLE_GUI'] and ENV['PCSD_DISABLE_GUI'].downcase == 'true'
   )
-  PCS = get_pcs_path(File.expand_path(File.dirname(__FILE__)))
+  PCS = get_pcs_path()
   logger = File.open("/var/log/pcsd/pcsd.log", "a+", 0600)
   STDOUT.reopen(logger)
   STDERR.reopen(logger)
@@ -568,7 +568,8 @@ already been added to pcsd.  You may not add two clusters with the same name int
       {
         :clustername => @cluster_name,
         :nodes => @nodes_rrp.join(';'),
-        :options => options.to_json
+        :options => options.to_json,
+        :encryption => params[:encryption],
       },
       true,
       nil,
@@ -778,7 +779,9 @@ already been added to pcsd.  You may not add two clusters with the same name int
     if @nodes == []
       redirect '/manage/'
     end
-    @resource_agents = get_resource_agents_avail(auth_user, params)
+    @resource_agent_structures = get_resource_agents_avail(auth_user, params) \
+      .map{|agent_name| get_resource_agent_name_structure(agent_name)} \
+      .select{|structure| structure != nil}
     @stonith_agents = get_stonith_agents_avail(auth_user, params)
     erb :nodes, :layout => :main
   end
@@ -967,7 +970,7 @@ already been added to pcsd.  You may not add two clusters with the same name int
       else
         definition = JSON.parse(out)
       end
-  
+
       definition.each { |name, prop|
         prop['value'] = properties[name]
       }
