@@ -72,10 +72,12 @@ Pcs = Ember.Application.createWithMixins({
     }
     var out =
       '<table class="darkdatatable"><tr><th>OPTION</th><th>VALUE</th></tr>\n';
-    var banned_options = ["SBD_OPTS", "SBD_WATCHDOG_DEV", "SBD_PACEMAKER"];
+    var banned_options = [
+      "SBD_OPTS", "SBD_WATCHDOG_DEV", "SBD_PACEMAKER", "SBD_DEVICE"
+    ];
     $.each(this.get("sbd_config"), function(opt, val) {
       if (banned_options.indexOf(opt) == -1) {
-        out += '<tr><td>' + opt + '</td><td>' + val + '</td></tr>\n';
+        out += '<tr><td>' + htmlEncode(opt) + '</td><td>' + htmlEncode(val) + '</td></tr>\n';
       }
     });
     return out + '</table>';
@@ -273,7 +275,7 @@ Pcs.GroupSelectorComponent = Ember.Component.extend({
   group_list: [],
   group_select_content: function() {
     var list = [];
-    $.each(this.get("group_list"), function(_, group) {
+    $.each(this.getWithDefault("group_list", []), function(_, group) {
       list.push({
         name: group,
         value: group
@@ -809,7 +811,7 @@ Pcs.resourcesContainer.reopen({
   groups_enum: function() {
     var self = this;
     var res = [];
-    $.each(self.get("group_list"), function(_, group) {
+    $.each(self.getWithDefault("group_list", []), function(_, group) {
       res.push({
         name: group,
         value: group
@@ -879,7 +881,7 @@ Pcs.ResourceObj = Ember.Object.extend({
   }.property("status_val"),
   show_status: function() {
     return '<span style="' + this.get('status_style') + '">'
-      + this.get('status') + (this.get("is_unmanaged") ? " (unmanaged)" : "")
+      + htmlEncode(this.get('status')) + (this.get("is_unmanaged") ? " (unmanaged)" : "")
       + '</span>';
   }.property("status_style", "disabled"),
   status_class: function() {
@@ -1226,7 +1228,8 @@ Pcs.ResourceAgentParameter = Ember.Object.extend({
     if (longdesc) desc.push(longdesc);
     if (def_val) desc.push("Default value: " + def_val);
     return desc.join("<br /><br />");
-  }.property("longdesc", "shortdesc", "default")
+  }.property("longdesc", "shortdesc", "default"),
+  pcs_deprecated_warning: ""
 });
 
 Pcs.ResourceAgent = Ember.Object.extend({
