@@ -102,9 +102,8 @@ function create_group() {
     return;
   }
   var not_primitives = resource_list.filter(function(resource_id) {
-    return !Pcs.resourcesContainer.get_resource_by_id(resource_id).get(
-      "is_primitive"
-    );
+    var resource_obj = Pcs.resourcesContainer.get_resource_by_id(resource_id);
+    return !(resource_obj && resource_obj.get("is_primitive"));
   });
   if (not_primitives.length != 0) {
     alert("Members of group have to be primitive resources. These resources" +
@@ -209,6 +208,29 @@ function add_node_dialog() {
   });
 }
 
+function add_sbd_device_textbox() {
+  var max_count = 3;
+  var device_inputs = $("#add_node_selector .add_node_sbd_device");
+  var count = device_inputs.length;
+  if (count < max_count) {
+    $(device_inputs[count-1]).after(
+      '<tr class="add_node_sbd_device"> \
+        <td>&nbsp;</td> \
+        <td> \
+          <input type="text" name="devices[]" /> \
+          <a href="#" onclick="sbd_device_remove_textbox(this); return false;"> \
+            (-) \
+          </a> \
+        </td> \
+      </tr>'
+    )
+  }
+}
+
+function sbd_device_remove_textbox(obj) {
+  $(obj).parents(".add_node_sbd_device").remove();
+}
+
 function checkAddingNode(){
   var nodeName = $("#add_node").children("form").find("[name='new_nodename']").val().trim();
   if (nodeName == "") {
@@ -822,7 +844,7 @@ function auth_nodes_dialog(unauth_nodes, callback_success, callback_success_one)
 
   dialog_obj.find('#auth_nodes_list').empty();
   unauth_nodes.forEach(function(node) {
-    dialog_obj.find('#auth_nodes_list').append("\t\t\t<tr><td>" + node + '</td><td><input type="password" name="' + node + '-pass"></td></tr>\n');
+    dialog_obj.find('#auth_nodes_list').append("\t\t\t<tr><td>" + htmlEncode(node) + '</td><td><input type="password" name="' + htmlEncode(node) + '-pass"></td></tr>\n');
   });
 
 }
@@ -1058,7 +1080,7 @@ function update_create_cluster_dialog(nodes, version_info) {
     ajax_wrapper({
       type: "POST",
       url: "/manage/newcluster",
-      timeout: pcs_timeout,
+      timeout: 60*1000,
       data: $('#create_new_cluster_form').serialize(),
       success: function(data) {
         if (data) {
@@ -1068,7 +1090,17 @@ function update_create_cluster_dialog(nodes, version_info) {
         Pcs.update();
       },
       error: function (xhr, status, error) {
-        alert(xhr.responseText);
+        var err_msg = "";
+        if ((status == "timeout") || ($.trim(error) == "timeout")) {
+          err_msg = (
+            "Operation takes longer to complete than expected. " +
+            "It may continue running in the background. Later, you can try " +
+            "to add this cluster as existing one."
+          );
+        } else {
+          err_msg = xhr.responseText;
+        }
+        alert(err_msg);
         $("#create_cluster_submit_btn").button("option", "disabled", false);
       }
     });
