@@ -9,11 +9,16 @@ require 'tempfile'
 
 require 'pcs.rb'
 require 'resource.rb'
+require 'settings.rb'
 require 'config.rb'
 require 'cfgsync.rb'
 require 'cluster_entity.rb'
 require 'permissions.rb'
 require 'auth.rb'
+require 'pcsd_file'
+require 'pcsd_remove_file'
+require 'pcsd_action_command'
+require 'pcsd_exchange_format.rb'
 
 # Commands for remote access
 def remote(params, request, auth_user)
@@ -36,6 +41,7 @@ def remote(params, request, auth_user)
       :set_configs => method(:set_configs),
       :set_certs => method(:set_certs),
       :pcsd_restart => method(:remote_pcsd_restart),
+      :pcsd_instance_signature => method(:pcsd_instance_signature),
       :get_permissions => method(:get_permissions_remote),
       :set_permissions => method(:set_permissions_remote),
       :cluster_start => method(:cluster_start),
@@ -86,6 +92,9 @@ def remote(params, request, auth_user)
       :booth_set_config => method(:booth_set_config),
       :booth_save_files => method(:booth_save_files),
       :booth_get_config => method(:booth_get_config),
+      :put_file => method(:put_file),
+      :remove_file => method(:remove_file),
+      :manage_services => method(:manage_services),
 
   }
   remote_cmd_with_pacemaker = {
@@ -763,7 +772,14 @@ end
 
 def remote_pcsd_restart(params, request, auth_user)
   pcsd_restart()
-  return [200, 'success']
+  return JSON.generate({
+    :success => true,
+    :instance_signature => DAEMON_INSTANCE_SIGNATURE,
+  })
+end
+
+def pcsd_instance_signature(params, request, auth_user)
+  return [200, DAEMON_INSTANCE_SIGNATURE]
 end
 
 def get_sw_versions(params, request, auth_user)
@@ -791,6 +807,12 @@ def remote_node_available(params, request, auth_user)
       :pacemaker_remote => true,
     })
   end
+  if pacemaker_running?()
+    return JSON.generate({
+      :node_available => false,
+      :pacemaker_running => true,
+    })
+  end
   return JSON.generate({:node_available => true})
 end
 
@@ -808,8 +830,12 @@ def remote_add_node(params, request, auth_user, all=false)
     if params[:new_ring1addr] != nil
       node += ',' + params[:new_ring1addr]
     end
+    device_list = []
+    if params[:devices].kind_of?(Array)
+      device_list = params[:devices]
+    end
     retval, output = add_node(
-      auth_user, node, all, auto_start, params[:watchdog]
+      auth_user, node, all, auto_start, params[:watchdog], device_list
     )
   end
 
@@ -938,9 +964,12 @@ def setup_cluster(params, request, auth_user)
   end
   nodes_options = nodes + options
   nodes_options += options_udp if transport_udp
+  if ['0', '1'].include?(params[:encryption])
+      nodes_options << "--encryption=#{params[:encryption]}"
+  end
   stdout, stderr, retval = run_cmd(
-    auth_user, PCS, "cluster", "setup", "--enable", "--start",
-    "--name", params[:clustername], *nodes_options
+    auth_user, PCS, "cluster", "setup", "--enable", "--start", "--async",
+    "--name",  params[:clustername], *nodes_options
   )
   if retval != 0
     return [
@@ -1437,6 +1466,13 @@ def update_resource (params, request, auth_user)
       end
       resource_group = params[:resource_group]
     end
+    if params[:resource_type] == "ocf:pacemaker:remote"
+      # Workaround for Error: this command is not sufficient for create remote
+      # connection, use 'pcs cluster node add-remote', use --force to override.
+      # It is not possible to specify meta attributes so we don't need to take
+      # care of those.
+      cmd << "--force"
+    end
     out, stderr, retval = run_cmd(auth_user, *cmd)
     if retval != 0
       return JSON.generate({"error" => "true", "stderr" => stderr, "stdout" => out})
@@ -1545,8 +1581,7 @@ def get_avail_resource_agents(params, request, auth_user)
   if not allowed_for_local_cluster(auth_user, Permissions::READ)
     return 403, 'Permission denied'
   end
-  agents = getResourceAgents(auth_user)
-  return JSON.generate(agents)
+  return JSON.generate(getResourceAgents(auth_user).map{|a| [a, {}]}.to_h)
 end
 
 def get_avail_fence_agents(params, request, auth_user)
@@ -2323,6 +2358,21 @@ def check_sbd(param, request, auth_user)
       :exist => File.exist?(watchdog)
     }
   end
+  begin
+    device_list = JSON.parse(param[:device_list])
+    if device_list and device_list.respond_to?('each')
+      out[:device_list] = []
+      device_list.each { |device|
+        out[:device_list] << {
+          :path => device,
+          :exist => File.exists?(device),
+          :block_device => File.blockdev?(device),
+        }
+      }
+    end
+  rescue JSON::ParserError
+    return [400, 'Invalid input data format']
+  end
   return [200, JSON.generate(out)]
 end
 
@@ -2341,18 +2391,14 @@ def set_sbd_config(param, request, auth_user)
     file.flock(File::LOCK_EX)
     file.write(config)
   rescue => e
-    msg = "Unable to save SBD configuration: #{e}"
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Unable to save SBD configuration: #{e}")
   ensure
     if file
       file.flock(File::LOCK_UN)
       file.close()
     end
   end
-  msg = 'SBD configuration saved.'
-  $logger.info(msg)
-  return [200, msg]
+  return pcsd_success('SBD configuration saved.')
 end
 
 def get_sbd_config(param, request, auth_user)
@@ -2366,9 +2412,7 @@ def get_sbd_config(param, request, auth_user)
     file.flock(File::LOCK_SH)
     out = file.readlines()
   rescue => e
-    msg = "Unable to get SBD configuration: #{e}"
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Unable to get SBD configuration: #{e}")
   ensure
     if file
       file.flock(File::LOCK_UN)
@@ -2383,13 +2427,9 @@ def sbd_disable(param, request, auth_user)
     return 403, 'Permission denied'
   end
   if disable_service(get_sbd_service_name())
-    msg = 'SBD disabled'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('SBD disabled')
   else
-    msg = 'Disabling SBD failed'
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Disabling SBD failed")
   end
 end
 
@@ -2398,13 +2438,9 @@ def sbd_enable(param, request, auth_user)
     return 403, 'Permission denied'
   end
   if enable_service(get_sbd_service_name())
-    msg = 'SBD enabled'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('SBD enabled')
   else
-    msg = 'Enabling SBD failed'
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Enabling SBD failed")
   end
 end
 
@@ -2580,13 +2616,9 @@ def qdevice_client_disable(param, request, auth_user)
     return 403, 'Permission denied'
   end
   if disable_service('corosync-qdevice')
-    msg = 'corosync-qdevice disabled'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('corosync-qdevice disabled')
   else
-    msg = 'Disabling corosync-qdevice failed'
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Disabling corosync-qdevice failed")
   end
 end
 
@@ -2595,17 +2627,11 @@ def qdevice_client_enable(param, request, auth_user)
     return 403, 'Permission denied'
   end
   if not is_service_enabled?('corosync')
-    msg = 'corosync is not enabled, skipping'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('corosync is not enabled, skipping')
   elsif enable_service('corosync-qdevice')
-    msg = 'corosync-qdevice enabled'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('corosync-qdevice enabled')
   else
-    msg = 'Enabling corosync-qdevice failed'
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Enabling corosync-qdevice failed")
   end
 end
 
@@ -2614,13 +2640,9 @@ def qdevice_client_stop(param, request, auth_user)
     return 403, 'Permission denied'
   end
   if stop_service('corosync-qdevice')
-    msg = 'corosync-qdevice stopped'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('corosync-qdevice stopped')
   else
-    msg = 'Stopping corosync-qdevice failed'
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Stopping corosync-qdevice failed")
   end
 end
 
@@ -2629,17 +2651,11 @@ def qdevice_client_start(param, request, auth_user)
     return 403, 'Permission denied'
   end
   if not is_service_running?('corosync')
-    msg = 'corosync is not running, skipping'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('corosync is not running, skipping')
   elsif start_service('corosync-qdevice')
-    msg = 'corosync-qdevice started'
-    $logger.info(msg)
-    return [200, msg]
+    return pcsd_success('corosync-qdevice started')
   else
-    msg = 'Starting corosync-qdevice failed'
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Starting corosync-qdevice failed")
   end
 end
 
@@ -2686,98 +2702,118 @@ def unmanage_resource(param, request, auth_user)
 end
 
 def booth_set_config(params, request, auth_user)
-  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
-    return 403, 'Permission denied'
-  end
   begin
-    unless params[:data_json]
-      return [400, "Missing required parameter 'data_json'"]
+    check_permissions(auth_user, Permissions::WRITE)
+    data = check_request_data_for_json(params, auth_user)
+
+    PcsdExchangeFormat::validate_item_map_is_Hash('files', data)
+    PcsdExchangeFormat::validate_item_is_Hash('file', :config, data[:config])
+    if data[:authfile]
+      PcsdExchangeFormat::validate_item_is_Hash('file', :config, data[:config])
     end
-    data = JSON.parse(params[:data_json], {:symbolize_names => true})
-  rescue JSON::ParserError
-    return [400, 'Invalid input data format']
-  end
-  config = data[:config]
-  authfile = data[:authfile]
-  return [400, 'Invalid input data format'] unless (
-    config and config[:name] and config[:data]
-  )
-  return [400, 'Invalid input data format'] if (
-    authfile and (not authfile[:name] or not authfile[:data])
-  )
-  begin
-    write_booth_config(config[:name], config[:data])
-    if authfile
-      write_booth_authfile(authfile[:name], authfile[:data])
+
+    action_results = {
+      :config => PcsdExchangeFormat::run_action(
+        PcsdFile::TYPES,
+        "file",
+        :config,
+        data[:config].merge({
+          :type => "booth_config",
+          :rewrite_existing => true
+        })
+      )
+    }
+
+    if data[:authfile]
+      action_results[:authfile] = PcsdExchangeFormat::run_action(
+        PcsdFile::TYPES,
+        "file",
+        :authfile,
+        data[:authfile].merge({
+          :type => "booth_authfile",
+          :rewrite_existing => true
+        })
+      )
     end
-  rescue InvalidFileNameException => e
-    return [400, "Invalid format of config/key file name '#{e.message}'"]
+
+    success_codes = [:written, :rewritten]
+    failed_results = action_results.select{|key, result|
+      !success_codes.include?(result[:code])
+    }
+
+    if failed_results.empty?
+      return pcsd_success('Booth configuration saved.')
+    end
+
+    return pcsd_error("Unable to save booth configuration: #{
+      failed_results.reduce([]){|memo, (key, result)|
+        memo << "#{key}: #{result[:code]}: #{result[:message]}"
+      }.join(";")
+    }")
+  rescue PcsdRequestException => e
+    return e.code, e.message
+  rescue PcsdExchangeFormat::Error => e
+    return 400, "Invalid input data format: #{e.message}"
   rescue => e
-    msg = "Unable to save booth configuration: #{e.message}"
-    $logger.error(msg)
-    return [400, msg]
+    return pcsd_error("Unable to save booth configuration: #{e.message}")
   end
-  msg = 'Booth configuration saved.'
-  $logger.info(msg)
-  return [200, msg]
 end
 
 def booth_save_files(params, request, auth_user)
-  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
-    return 403, 'Permission denied'
-  end
   begin
-    data = JSON.parse(params[:data_json], {:symbolize_names => true})
-    data.each { |file|
-      unless file[:name] and file[:data]
-        return [400, 'Invalid input data format']
-      end
-      if file[:name].include?('/')
-        return [400, "Invalid file name format '#{file[:name]}'"]
+    check_permissions(auth_user, Permissions::WRITE)
+    data = check_request_data_for_json(params, auth_user)
+    rewrite_existing = (
+      params.include?('rewrite_existing') || params.include?(:rewrite_existing)
+    )
+
+    action_results = Hash[data.each_with_index.map{|file, i|
+      PcsdExchangeFormat::validate_item_is_Hash('file', i, file)
+      [
+        i,
+        PcsdExchangeFormat::run_action(
+          PcsdFile::TYPES,
+          'file',
+          i,
+          file.merge({
+            :rewrite_existing => rewrite_existing,
+            :type => file[:is_authfile] ? "booth_authfile" : "booth_config"
+          })
+        )
+      ]
+    }]
+
+    results = {:existing => [], :saved => [], :failed => {}}
+
+    code_result_map = {
+      :written => :saved,
+      :rewritten => :saved,
+      :same_content => :saved,
+      :conflict => :existing,
+    }
+
+    action_results.each{|i, result|
+      name = data[i][:name]
+
+      if code_result_map.has_key?(result[:code])
+        results[code_result_map[result[:code]]] << name
+
+      elsif result[:code] == :unexpected
+        results[:failed][name] = result[:message]
+
+      else
+        results[:failed][name] = "Unknown process file result:"+
+          "code: '#{result[:code]}': message: '#{result[:message]}'"
+
       end
     }
-  rescue JSON::ParserError, NoMethodError
-    return [400, 'Invalid input data format']
+
+    return [200, JSON.generate(results)]
+  rescue PcsdRequestException => e
+    return e.code, e.message
+  rescue PcsdExchangeFormat::Error => e
+    return 400, "Invalid input data format: #{e.message}"
   end
-  rewrite_existing = (
-  params.include?('rewrite_existing') || params.include?(:rewrite_existing)
-  )
-
-  conflict_files = []
-  data.each { |file|
-    next unless File.file?(File.join(BOOTH_CONFIG_DIR, file[:name]))
-    if file[:is_authfile]
-      cur_data = read_booth_authfile(file[:name])
-    else
-      cur_data = read_booth_config(file[:name])
-    end
-    if cur_data != file[:data]
-      conflict_files << file[:name]
-    end
-  }
-
-  write_failed = {}
-  saved_files = []
-  data.each { |file|
-    next if conflict_files.include?(file[:name]) and not rewrite_existing
-    begin
-      if file[:is_authfile]
-        write_booth_authfile(file[:name], file[:data])
-      else
-        write_booth_config(file[:name], file[:data])
-      end
-      saved_files << file[:name]
-    rescue => e
-      msg = "Unable to save file (#{file[:name]}): #{e.message}"
-      $logger.error(msg)
-      write_failed[file[:name]] = e
-    end
-  }
-  return [200, JSON.generate({
-    :existing => conflict_files,
-    :saved => saved_files,
-    :failed => write_failed
-  })]
 end
 
 def booth_get_config(params, request, auth_user)
@@ -2825,6 +2861,73 @@ def booth_get_config(params, request, auth_user)
   end
 end
 
+def put_file(params, request, auth_user)
+  begin
+    check_permissions(auth_user, Permissions::WRITE)
+
+    files = check_request_data_for_json(params, auth_user)
+    PcsdExchangeFormat::validate_item_map_is_Hash('files', files)
+
+    return pcsd_success(
+      JSON.generate({"files" => Hash[files.map{|id, file_data|
+        PcsdExchangeFormat::validate_item_is_Hash('file', id, file_data)
+        [id, PcsdExchangeFormat::run_action(
+          PcsdFile::TYPES, 'file', id, file_data
+        )]
+      }]})
+    )
+  rescue PcsdRequestException => e
+    return e.code, e.message
+  rescue PcsdExchangeFormat::Error => e
+    return 400, "Invalid input data format: #{e.message}"
+  end
+end
+
+def remove_file(params, request, auth_user)
+  begin
+    check_permissions(auth_user, Permissions::WRITE)
+
+    files = check_request_data_for_json(params, auth_user)
+    PcsdExchangeFormat::validate_item_map_is_Hash('files', files)
+
+    return pcsd_success(
+      JSON.generate({"files" => Hash[files.map{|id, file_data|
+        PcsdExchangeFormat::validate_item_is_Hash('file', id, file_data)
+        [id, PcsdExchangeFormat::run_action(
+          PcsdRemoveFile::TYPES, 'file', id, file_data
+        )]
+      }]})
+    )
+  rescue PcsdRequestException => e
+    return e.code, e.message
+  rescue PcsdExchangeFormat::Error => e
+    return 400, "Invalid input data format: #{e.message}"
+  end
+end
+
+
+def manage_services(params, request, auth_user)
+  begin
+    check_permissions(auth_user, Permissions::WRITE)
+
+    actions = check_request_data_for_json(params, auth_user)
+    PcsdExchangeFormat::validate_item_map_is_Hash('actions', actions)
+
+    return pcsd_success(
+      JSON.generate({"actions" => Hash[actions.map{|id, action_data|
+        PcsdExchangeFormat::validate_item_is_Hash("action", id, action_data)
+        [id, PcsdExchangeFormat::run_action(
+          PcsdActionCommand::TYPES, "action", id, action_data
+        )]
+      }]})
+    )
+  rescue PcsdRequestException => e
+    return e.code, e.message
+  rescue PcsdExchangeFormat::Error => e
+    return 400, "Invalid input data format: #{e.message}"
+  end
+end
+
 def _hash_to_argument_list(hash)
   result = []
   if hash.kind_of?(Hash)
@@ -2963,3 +3066,39 @@ def update_recipient(params, request, auth_user)
   end
   return [200, 'Recipient updated']
 end
+
+def pcsd_success(msg)
+  $logger.info(msg)
+  return [200, msg]
+end
+
+def pcsd_error(msg)
+  $logger.error(msg)
+  return [400, msg]
+end
+
+class PcsdRequestException < StandardError
+  attr_accessor :code
+
+  def initialize(message = nil, code = 400)
+    super(message)
+    self.code = code
+  end
+end
+
+def check_permissions(auth_user, permission)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    raise PcsdRequestException.new('Permission denied', 403)
+  end
+end
+
+def check_request_data_for_json(params, auth_user)
+  unless params[:data_json]
+    raise PcsdRequestException.new("Missing required parameter 'data_json'")
+  end
+  begin
+    return JSON.parse(params[:data_json], {:symbolize_names => true})
+  rescue JSON::ParserError
+    raise PcsdRequestException.new('Invalid input data format')
+  end
+end
