@@ -2,6 +2,7 @@ require 'rubygems'
 require 'webrick'
 require 'webrick/https'
 require 'openssl'
+gem 'rack', '< 2.0.0'
 require 'rack'
 require 'socket'
 
@@ -65,35 +66,44 @@ def run_server(server, webrick_options, secondary_addrs)
   primary_addr = webrick_options[:BindAddress]
   port = webrick_options[:Port]
 
-  ciphers = 'DEFAULT:!RC4:!3DES:@STRENGTH!'
+  ciphers = 'DEFAULT:!RC4:!3DES:@STRENGTH'
   ciphers = ENV['PCSD_SSL_CIPHERS'] if ENV['PCSD_SSL_CIPHERS']
   # no need to validate ciphers, ssl context will validate them for us
 
   $logger.info("Listening on #{primary_addr} port #{port}")
-  server.run(Sinatra::Application, webrick_options) { |server_instance|
-    # configure ssl options
-    server_instance.ssl_context.ciphers = ciphers
-    # set listening addresses
-    secondary_addrs.each { |addr|
-      $logger.info("Adding listener on #{addr} port #{port}")
-      server_instance.listen(addr, port)
-    }
-    # notify systemd we are running
-    if ISSYSTEMCTL
-      socket_name = ENV['NOTIFY_SOCKET']
-      if socket_name
-        if socket_name.start_with?('@')
-          # abstract namespace socket
-          socket_name[0] = "\0"
+  begin
+    server.run(Sinatra::Application, webrick_options) { |server_instance|
+      # configure ssl options
+      server_instance.ssl_context.ciphers = ciphers
+      # set listening addresses
+      secondary_addrs.each { |addr|
+        $logger.info("Adding listener on #{addr} port #{port}")
+        server_instance.listen(addr, port)
+      }
+      # notify systemd we are running
+      if ISSYSTEMCTL
+        if ENV['NOTIFY_SOCKET']
+          socket_name = ENV['NOTIFY_SOCKET'].dup
+          if socket_name.start_with?('@')
+            # abstract namespace socket
+            socket_name[0] = "\0"
+          end
+          $logger.info("Notifying systemd we are running (socket #{socket_name})")
+          sd_socket = Socket.new(Socket::AF_UNIX, Socket::SOCK_DGRAM)
+          sd_socket.connect(Socket.pack_sockaddr_un(socket_name))
+          sd_socket.send('READY=1', 0)
+          sd_socket.close()
         end
-        $logger.info("Notifying systemd we are running (socket #{socket_name})")
-        sd_socket = Socket.new(Socket::AF_UNIX, Socket::SOCK_DGRAM)
-        sd_socket.connect(Socket.pack_sockaddr_un(socket_name))
-        sd_socket.send('READY=1', 0)
-        sd_socket.close()
       end
-    end
-  }
+    }
+  rescue Errno::EADDRNOTAVAIL, Errno::EADDRINUSE => e
+    $logger.error 'Unable to bind to specified address(es), exiting'
+    $logger.error e.message
+    exit 1
+  rescue SocketError => e
+    $logger.error e.message
+    exit 1
+  end
 end
 
 if not File.exists?(CRT_FILE) or not File.exists?(KEY_FILE)
